from lino.projects.std.settings import *
#~ from lino_local import LocalLinoMixin


class Site(Site):

    title = "Babel Tutorial"

    #~ languages = ['en']
    #~ languages = ['de', 'fr']
    languages = ['en', 'fr']

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.system'
        yield 'lino_book.projects.babel_tutorial'

    def setup_menu(self, user_type, main):
        m = main.add_menu("products", "Products")
        m.add_action(self.modules.babel_tutorial.Products)
    
SITE = Site(globals())

DEBUG = True

# The DATABASES setting is the only thing you should take
# over from your original file:
#~ DATABASES = {
    #~ 'default': {
        #~ 'ENGINE': 'django.db.backends.sqlite3',
        #~ 'NAME': abspath(join(dirname(__file__),'test.db'))
    #~ }
#~ }



# -*- coding: UTF-8 -*-
# Copyright 2012-2014 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Content Management à la Lino.

Lino CMS is an example about how content management might look like
with Lino.  It is not yet being used on any production environment.


"""



# -*- coding: UTF-8 -*-
# Copyright 2012-2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""A special settings module to be used as DJANGO_SETTINGS_MODULE when
Sphinx generates the Lino docs.

It contains (almost) all Lino plugins, which makes no sense in
practice and would maybe raise errors if you try to initialize a
database or validate the models, but it is enough to have autodocs do
its job.  And that's all we want.

"""

import os

from lino.projects.std.settings import *


class Site(Site):

    demo_fixtures = 'std few_countries demo demo2'.split()

    verbose_name = "Lino Docs"

    project_name = 'lino_docs'

    project_model = 'contacts.Person'

    # languages = 'en de fr'
    languages = 'en de fr et nl pt-br es'

    user_types_module = 'lino.core.user_types'

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()

        yield 'lino.modlib.system'
        # yield 'lino.modlib.gfks'
        yield 'lino.modlib.users'
        yield 'lino.modlib.changes'
        yield 'lino.modlib.notify'
        yield 'lino.modlib.languages'
        yield 'lino_xl.lib.countries'
        # yield 'lino_xl.lib.properties'
        yield 'lino_xl.lib.contacts'
        # yield 'lino_xl.lib.addresses'
        # yield 'lino_xl.lib.humanlinks'  # requires Person to be Born

        yield 'lino.modlib.uploads'
        # yield 'lino_xl.lib.notes'
        # yield 'lino_xl.lib.outbox'
        # yield 'lino_xl.lib.cal'
        # yield 'lino_xl.lib.extensible'
        # yield 'lino_xl.lib.reception'
        # yield 'lino_xl.lib.excerpts'
        yield 'lino_xl.lib.polls'
        # yield 'lino_xl.lib.vat'
        # yield 'lino_xl.lib.ledger'
        # yield 'lino_xl.lib.bevat'
        # yield 'lino_xl.lib.cv'
        # yield 'lino_xl.lib.boards'
        # yield 'lino_xl.lib.postings'
        # yield 'lino_xl.lib.households'

        yield 'lino_xl.lib.concepts'
        # yield 'lino_xl.lib.pages'
        # yield 'lino_xl.lib.beid'
        yield 'lino.modlib.tinymce'
        yield 'lino.modlib.export_excel'

    def setup_plugins(self):
        super(Site, self).setup_plugins()
        self.plugins.countries.configure(country_code='BE')

    # def do_site_startup(self):
    #     # lino_xl.lib.reception requires some workflow to be imported
    #     from lino_xl.lib.cal.workflows import feedback
    #     super(Site, self).do_site_startup()




# -*- coding: UTF-8 -*-
# Copyright 2015-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Settings for providing readonly public access to the site. This
does not use :mod:`lino.modlib.extjs` but :mod:`lino.modlib.bootstrap3`.

"""

import datetime

from ..settings import *


class Site(Site):
    the_demo_date = datetime.date(2015, 5, 23)
    languages = "en de fr"
    readonly = True
    # default_user = 'anonymous'


team_db = DATABASES

SITE = Site(globals())

DATABASES = team_db

DEBUG = True




# -*- coding: utf-8 -*-
# Copyright 2017-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Basic tests.

This module is part of the Lino test suite. You can test only this
module by issuing either::

  $ go watch
  $ python manage.py test tests.test_basics

or::

  $ go lino
  $ python setup.py test -s tests.ProjectsTests.test_watch


"""

from __future__ import unicode_literals
from __future__ import print_function

import six
from django.conf import settings

from lino.utils.djangotest import RemoteAuthTestCase

from lino.core.callbacks import popCallBack, applyCallbackChoice
class QuickTest(RemoteAuthTestCase):

    # fixtures = ['demo', 'demo2']
    maxDiff = None

    def test_this(self):

        from lino.api import rt
        from lino.core.renderer import TestRenderer
        
        UserTypes = rt.models.users.UserTypes        
        rt.models.users.User(
            username="robin", user_type=UserTypes.admin).save()
        
        ses = rt.login('robin', renderer=TestRenderer(settings.SITE.kernel.default_ui))

        s = ses.show('changes.Changes')
        self.assertEqual(s, "No data to display")


        rr = rt.models.contacts.Companies.required_roles
        self.assertTrue(ses.user.user_type.role.satisfies_requirement(rr))

        # We create a new organization:

        url = '/api/contacts/Companies'
        data = dict(an='submit_insert', name='My pub')
        res = self.post_json_dict('robin', url, data)
        self.assertEqual(
            res.message, 'Organization "My pub" has been created.')

        s = ses.show('changes.Changes',
                     column_names="id type master object diff")
        # print(s)
        expected = """\
==== ============= ========== ========================================== ===============================================
 ID   Change Type   Master     Object                                     Changes
---- ------------- ---------- ------------------------------------------ -----------------------------------------------
 1    Create        *My pub*   `My pub </api/contacts/Companies/100>`__   Company(id=100,name='My pub',partner_ptr=100)
==== ============= ========== ========================================== ===============================================
"""
        
        self.assertEqual(s, expected)
        
        url = '/api/contacts/Companies/100'
        data = "an=submit_detail&name=Our%20pub"
        res = self.put_json_dict('robin', url, data)
        self.assertEqual(
            res.message, 'Organization "Our pub" has been updated.')

        output = ses.show('changes.Changes', column_names="id type master object diff")
        # print(output)
        expected = """\
==== ============= =========== =========================================== ===============================================
 ID   Change Type   Master      Object                                      Changes
---- ------------- ----------- ------------------------------------------- -----------------------------------------------
 2    Update        *Our pub*   `Our pub </api/contacts/Companies/100>`__   name : 'My pub' --> 'Our pub'
 1    Create        *Our pub*   `Our pub </api/contacts/Companies/100>`__   Company(id=100,name='My pub',partner_ptr=100)
==== ============= =========== =========================================== ===============================================
"""
        self.assertEqual(output, expected)

        # We add an entry:

        url = '/api/entries/Entries'
        data = dict(an='submit_insert', subject='test', companyHidden=100)
        res = self.post_json_dict('robin', url, data)
        if six.PY2:
            self.assertEqual(
                res.message, 'Entry "Entry object" has been created.')
        else:
            self.assertEqual(
                res.message, 'Entry "Entry object (1)" has been created.')

        expected = """\
==== ============= =========== =============================================== ===============================================
 ID   Change Type   Master      Object                                          Changes
---- ------------- ----------- ----------------------------------------------- -----------------------------------------------
 3    Create        *Our pub*   `Entry object (1) </api/entries/Entries/1>`__   Entry(id=1,user=1,subject='test',company=100)
 2    Update        *Our pub*   `Our pub </api/contacts/Companies/100>`__       name : 'My pub' --> 'Our pub'
 1    Create        *Our pub*   `Our pub </api/contacts/Companies/100>`__       Company(id=100,name='My pub',partner_ptr=100)
==== ============= =========== =============================================== ===============================================
"""

        output = ses.show('changes.Changes',
                     column_names="id type master object diff")
        # print(output)
        self.assertEqual(output, expected)
        
        
        # Now we delete the entry:

        url = '/api/entries/Entries/1'
        data = dict(an='delete_selected', sr=1)
        res = self.get_json_dict('robin', url, data)
        if six.PY2:
            self.assertEqual(
            res.message, """\
You are about to delete 1 Entry:
Entry object
Are you sure ?""")
        else:
            self.assertEqual(
                res.message, """\
You are about to delete 1 Entry:
Entry object (1)
Are you sure ?""")

        
        
        # We answer "yes":
        applyCallbackChoice(res, data, "yes")
        # url = "/callbacks/{0}/yes".format(res['xcallback']['id'])
        # print(data)
        res = self.get_json_dict('robin', url, data)
        # print(res)
        # r = test_client.get(url)
        self.assertEqual(res.success, True)
        self.assertEqual(res.record_deleted, True)
        

        expected = """\
==== ============= =========== =========================================== ===============================================
 ID   Change Type   Master      Object                                      Changes
---- ------------- ----------- ------------------------------------------- -----------------------------------------------
 4    Delete        *Our pub*                                               Entry(id=1,user=1,subject='test',company=100)
 3    Create        *Our pub*                                               Entry(id=1,user=1,subject='test',company=100)
 2    Update        *Our pub*   `Our pub </api/contacts/Companies/100>`__   name : 'My pub' --> 'Our pub'
 1    Create        *Our pub*   `Our pub </api/contacts/Companies/100>`__   Company(id=100,name='My pub',partner_ptr=100)
==== ============= =========== =========================================== ===============================================
"""
        output = ses.show('changes.Changes',
                     column_names="id type master object diff")
        # print(output)
        self.assertEqual(output, expected)

        # Note how the `object` column of the first two rows in above
        # table is empty. That's because the entry object has been
        # deleted, so it does no longer exist in the database and Lino
        # cannot point to it. But note also that `object` is a
        # "nullable Generic ForeignKey", the underlying fields
        # `object_id` and `object_type` still contain their
        # values. Here is the same table with "Object" split into its
        # components:

        expected = """\
==== ============= =========== ============== =========== ===============================================
 ID   Change Type   Master      Object type    object id   Changes
---- ------------- ----------- -------------- ----------- -----------------------------------------------
 4    Delete        *Our pub*   Entry          1           Entry(id=1,user=1,subject='test',company=100)
 3    Create        *Our pub*   Entry          1           Entry(id=1,user=1,subject='test',company=100)
 2    Update        *Our pub*   Organization   100         name : 'My pub' --> 'Our pub'
 1    Create        *Our pub*   Organization   100         Company(id=100,name='My pub',partner_ptr=100)
==== ============= =========== ============== =========== ===============================================
"""
        output = ses.show('changes.Changes',
                     column_names="id type master object_type object_id diff")
        # print(output)
        self.assertEqual(output, expected)


# Until 20150626 only the
# :attr:`object<lino.modlib.changes.models.Change.object>` was nullable,
# not the :attr:`master<lino.modlib.changes.models.Change.master>`.  But
# now you can also delete the master, and all change records will still
# remain:

        url = '/api/contacts/Companies/100'
        data = dict(an='delete_selected', sr=100)
        res = self.get_json_dict('robin', url, data)
        applyCallbackChoice(res, data, "yes")
        # url = "/callbacks/{0}/yes".format(res.xcallback['id'])
        self.get_json_dict('robin', url, data)
        
        expected = """\
==== ============= ======== ======== ================================================
 ID   Change Type   Master   Object   Changes
---- ------------- -------- -------- ------------------------------------------------
 5    Delete                          Company(id=100,name='Our pub',partner_ptr=100)
 4    Delete                          Entry(id=1,user=1,subject='test',company=100)
 3    Create                          Entry(id=1,user=1,subject='test',company=100)
 2    Update                          name : 'My pub' --> 'Our pub'
 1    Create                          Company(id=100,name='My pub',partner_ptr=100)
==== ============= ======== ======== ================================================
"""
        output = ses.show('changes.Changes',
                     column_names="id type master object diff")
        # print(output)
        self.assertEqual(output, expected)
        

        # Of course these change records are now considered broken GFKs:

        expected = """\
================================================ ================================= ============================================================= ========
 Database model                                   Database object                   Message                                                       Action
------------------------------------------------ --------------------------------- ------------------------------------------------------------- --------
 `Change </api/contenttypes/ContentTypes/14>`__   `#1 </api/changes/Changes/1>`__   Invalid primary key 100 for contacts.Partner in `master_id`   clear
 `Change </api/contenttypes/ContentTypes/14>`__   `#2 </api/changes/Changes/2>`__   Invalid primary key 100 for contacts.Partner in `master_id`   clear
 `Change </api/contenttypes/ContentTypes/14>`__   `#3 </api/changes/Changes/3>`__   Invalid primary key 100 for contacts.Partner in `master_id`   clear
 `Change </api/contenttypes/ContentTypes/14>`__   `#4 </api/changes/Changes/4>`__   Invalid primary key 100 for contacts.Partner in `master_id`   clear
 `Change </api/contenttypes/ContentTypes/14>`__   `#5 </api/changes/Changes/5>`__   Invalid primary key 100 for contacts.Partner in `master_id`   clear
 `Change </api/contenttypes/ContentTypes/14>`__   `#1 </api/changes/Changes/1>`__   Invalid primary key 100 for contacts.Company in `object_id`   clear
 `Change </api/contenttypes/ContentTypes/14>`__   `#2 </api/changes/Changes/2>`__   Invalid primary key 100 for contacts.Company in `object_id`   clear
 `Change </api/contenttypes/ContentTypes/14>`__   `#3 </api/changes/Changes/3>`__   Invalid primary key 1 for entries.Entry in `object_id`        clear
 `Change </api/contenttypes/ContentTypes/14>`__   `#4 </api/changes/Changes/4>`__   Invalid primary key 1 for entries.Entry in `object_id`        clear
 `Change </api/contenttypes/ContentTypes/14>`__   `#5 </api/changes/Changes/5>`__   Invalid primary key 100 for contacts.Company in `object_id`   clear
================================================ ================================= ============================================================= ========
"""
        output = ses.show('gfks.BrokenGFKs')
        # print(output)
        self.assertEqual(output, expected)
        
        # There open questions regarding these change records:

        # - Do we really never want to remove them? Do we really want a nullable
        #   master field? Should this option be configurable?
        # - How to tell :class:`lino.modlib.gfks.models.BrokenGFKs` to
        #   differentiate them from ?
        # - Should :meth:`get_broken_generic_related
        #   <lino.core.kernel.Kernel.get_broken_generic_related>` suggest to
        #   "clear" nullable GFK fields?




# -*- coding: UTF-8 -*-
# Copyright 2012-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from lino.projects.std.settings import *


class Site(Site):
    title = "Lino Mini 9"
    project_model = 'contacts.Person'
    # project_model = 'projects.Project'
    languages = 'en et fr'
    user_types_module = 'lino_xl.lib.xl.user_types'

    demo_fixtures = """std demo demo2 checkdata""".split()

    default_build_method = 'weasy2pdf'

    # def setup_plugins(self):
    #     super(Site, self).setup_plugins()
    #     self.plugins.comments.configure(
    #         commentable_model='contacts.Partner')

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        # yield 'lino.modlib.users'
        yield 'lino_xl.lib.excerpts'
        yield 'lino_book.projects.min9.modlib.contacts'
        yield 'lino_xl.lib.addresses'
        yield 'lino_xl.lib.reception'
        yield 'lino_xl.lib.courses'
        # yield 'lino.modlib.sepa'
        yield 'lino_xl.lib.notes'
        # yield 'lino_xl.lib.projects'
        yield 'lino_xl.lib.humanlinks'
        yield 'lino_xl.lib.households'
        yield 'lino_xl.lib.calview'
        # yield 'lino_xl.lib.extensible'
        yield 'lino_xl.lib.pages'
        yield 'lino.modlib.export_excel'
        yield 'lino_xl.lib.dupable_partners'
        yield 'lino.modlib.checkdata'
        yield 'lino.modlib.tinymce'
        # yield 'lino.modlib.wkhtmltopdf'
        yield 'lino.modlib.weasyprint'
        yield 'lino_xl.lib.appypod'
        yield 'lino.modlib.notify'
        yield 'lino.modlib.changes'
        yield 'lino.modlib.comments'
        yield 'lino.modlib.uploads'

    # def setup_actions(self):
    #     super(Site, self).setup_actions()
    #     partners = self.modules.contacts
    #     from lino.core.merge import MergeAction
    #     for m in (partners.Person, partners.Organisation):
    #         m.define_action(merge_row=MergeAction(m))

    # migration_module = 'lino_book.projects.min9.db_migrations'



# -*- coding: UTF-8 -*-
# Copyright 2015-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from django.db import models

class Restaurant(models.Model):
    name = models.CharField(max_length=255)


class Bar(Restaurant):
    bar_restaurant = models.OneToOneField(
        Restaurant, parent_link=True, on_delete=models.CASCADE)
    min_age = models.IntegerField()


class Pizzeria(Restaurant):
    pizzeria_restaurant = models.OneToOneField(
        Restaurant, parent_link=True, on_delete=models.CASCADE)
    specialty = models.CharField(max_length=255)


class PizzeriaBar(Bar, Pizzeria):
    pizza_bar_specific_field = models.CharField(max_length=255)



# -*- coding: utf-8 -*-
# Copyright 2016-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Runs some tests about the disable-delete handler and cascading deletes.

You can run only these tests by issuing::

  $ go team
  $ python manage.py test tests.test_ddh

Or::

  $ go book
  $ python setup.py test tests.test_ddh

"""

from __future__ import unicode_literals
from __future__ import print_function

from django.core.exceptions import ValidationError
from lino.utils.djangotest import RemoteAuthTestCase
from lino.api import rt

from lino.utils.instantiator import create_row as create
    

class DDHTests(RemoteAuthTestCase):
    maxDiff = None

    def test01(self):
        from lino.modlib.users.choicelists import UserTypes
        Ticket = rt.models.tickets.Ticket
        User = rt.models.users.User
        Subscription = rt.models.tickets.Subscription
        Site = rt.models.tickets.Site
        # ContentType = rt.models.contenttypes.ContentType
        # ct_Ticket = ContentType.objects.get_for_model(Ticket)

        site = create(Site, name='project')
        robin = create(User, username='robin',
                       first_name="Robin",
                       user_type=UserTypes.admin,
                       language="en")
        create(Subscription, site=site, user=robin)

        def createit():
            return create(Ticket, summary="Test", user=robin, site=site)

        #
        # If there are no vetos, user can ask to delete it
        #
        ticket = createit()
        ticket.delete()

        ticket = createit()

        # we cannot delete the user because a ticket refers to it:

        try:
            robin.delete()
            self.fail("Expected veto")
        except Warning as e:
            self.assertEqual(
                str(e), "Cannot delete User Robin "
                "because 1 Tickets refer to it.")

        
        self.assertEqual(Subscription.objects.count(), 1)
        self.assertEqual(Ticket.objects.count(), 1)

        # when we have deleted the ticket, deleting the user works
        # because the subscription is deleted in cascade:
        
        ticket.delete()
        robin.delete()
        self.assertEqual(Subscription.objects.count(), 0)
        self.assertEqual(Ticket.objects.count(), 0)
        self.assertEqual(User.objects.count(), 0)



# -*- coding: UTF-8 -*-
# Copyright 2014-2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
This is the main module of the `estref` project.
This is like :ref:`belref`, but for Estonia instead of Belgium.
"""




#!/usr/bin/env python
# To use this,you need to install line_profiler
# pip install line_profiler
# Add @profile to to method we need to profile.
# And run kernprof -lv lino_profiler.py


import os
os.environ.setdefault(
    'DJANGO_SETTINGS_MODULE',
    'lino_book.projects.team.settings.demo'
)
import django
django.setup()
from lino.api.doctest import *
from django.core.servers.basehttp import get_internal_wsgi_application
from django.test.client import RequestFactory
user = rt.models.users.User.objects.get(username="robin")
# from lino.modlib.extjs.views import AdminIndex

request_factory = RequestFactory()
# user = User.objects.get()
request = request_factory.get('/')
request.session = {}
request.user = user
request._cached_user = user

app = get_internal_wsgi_application()
app.get_response(request)

# view = AdminIndex.as_view()
# view(request).render()


# Generated by Django 2.2.4 on 2019-08-09 10:40

from django.db import migrations, models
import django.db.models.deletion
import lino.core.choicelists
import lino.core.fields
import lino.modlib.printing.choicelists


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contacts', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SiteConfig',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('default_build_method', lino.core.choicelists.ChoiceListField(blank=True, choicelist=lino.modlib.printing.choicelists.BuildMethods, choices=[(lino.modlib.printing.choicelists.LatexBuildMethod('latex'), 'LatexBuildMethod'), (lino.modlib.printing.choicelists.RtfBuildMethod('rtf'), 'RtfBuildMethod'), (lino.modlib.printing.choicelists.XmlBuildMethod('xml'), 'XmlBuildMethod')], help_text='The default build method to use when rendering printable documents.', max_length=50, null=True, verbose_name='Default build method')),
                ('simulate_today', models.DateField(blank=True, help_text='A constant user-defined date to be substituted as current\nsystem date.', null=True, verbose_name='Simulated date')),
                ('next_partner_id', models.IntegerField(blank=True, default=100, help_text='The next automatic id for any new partner.', null=True, verbose_name='Next partner id')),
                ('site_company', models.ForeignKey(blank=True, help_text='The organisation who runs this site.  This is used e.g. when\nprinting your address in certain documents or reports.  Or\nnewly created partners inherit the country of the site owner.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='site_company_sites', to='contacts.Company', verbose_name='Site owner')),
            ],
            options={
                'verbose_name': 'Site configuration',
                'abstract': False,
            },
            bases=(models.Model, lino.core.fields.TableRow),
        ),
    ]



# Copyright 2008-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""
See :doc:`/dev/combo/index`.
"""
from lino.ad import Plugin


class Plugin(Plugin):
    def setup_main_menu(self, site, user_type, m):
        m = m.add_menu(self.app_label, self.verbose_name)
        m.add_action('chooser.Countries')
        m.add_action('chooser.Cities')
        m.add_action('chooser.Contacts')



# -*- coding: UTF-8 -*-
# Copyright 2015-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from __future__ import unicode_literals
from builtins import str
from django.db import models
from lino.api import dd



class Person(dd.Model):
    
    name = models.CharField(max_length=50)

    def __str__(self):
        return self.name



class Place(dd.Model):
        
    name = models.CharField(max_length=50)
    owners = models.ManyToManyField(Person, related_name="owned_places")
    ceo = dd.ForeignKey(Person, related_name="managed_places")

    def __str__(self):
        if self.get_restaurant():
            if self.get_bar():
                what = "Restaurant & Bar "
            else:
                what = "Restaurant "
        elif self.get_bar():
            what = "Bar "
        else:
            what = ''
        return "%s %s(ceo=%s,owners=%s)" % (
            self.name, what, self.ceo,
            ','.join([str(o) for o in self.owners.all()]))

    def get_restaurant(self):
        try:
            return self.restaurant
        except Restaurant.DoesNotExist:
            return None

    def get_bar(self):
        try:
            return self.bar
        except Bar.DoesNotExist:
            return None



class Bar(dd.Model):
    
    place = dd.OneToOneField(Place)
    serves_alcohol = models.BooleanField(default=True)

    def __str__(self):
        if self.serves_alcohol:
            return self.place.name
        return "%s (no alcohol)" % self.place.name



class Restaurant(dd.Model):
        
    place = dd.OneToOneField(Place)
    serves_hot_dogs = models.BooleanField(default=False)
    cooks = models.ManyToManyField(Person)

    def __str__(self):
        return "%s (cooks=%s)" % (
            self.place.name,
            ','.join([str(o) for o in self.cooks.all()]))
    

class Visit(models.Model):
        
    allow_cascaded_delete = ['place']
    person = dd.ForeignKey(Person)
    place = dd.ForeignKey(Place)
    purpose = models.CharField(max_length=50)

    def __str__(self):
        return "%s visit by %s at %s" % (
            self.purpose, self.person, self.place.name)



class Meal(models.Model):
    allow_cascaded_delete = ['restaurant']
    person = dd.ForeignKey(Person)
    restaurant = dd.ForeignKey(Restaurant)
    what = models.CharField(max_length=50)

    def __str__(self):
        return "%s eats %s at %s" % (
            self.person, self.what, self.restaurant.name)





from lino.api import ad, _


class Plugin(ad.Plugin):
    verbose_name = _("Tables")

    def setup_main_menu(self, site, profile, m):
        m = m.add_menu(self.app_label, self.verbose_name)
        m.add_action('tables.Authors')
        m.add_action('tables.Books')

    



from lino.api import dd


class Persons(dd.Table):
    model = 'app.Person'
    column_names = 'name *'
    detail_layout = """
    id name
    VisitsByPerson
    MealsByPerson
    """


class Places(dd.Table):
    model = 'app.Place'
    detail_layout = """
    id name mti_navigator
    owners
    VisitsByPlace
    """


class Restaurants(dd.Table):
    model = 'app.Restaurant'
    detail_layout = """
    id name serves_hot_dogs mti_navigator
    owners cooks
    VisitsByPlace
    MealsByRestaurant
    """


class VisitsByPlace(dd.Table):
    model = 'app.Visit'
    master_key = 'place'
    column_names = 'person purpose'


class VisitsByPerson(dd.Table):
    model = 'app.Visit'
    master_key = 'person'
    column_names = 'place purpose'


class MealsByRestaurant(dd.Table):
    model = 'app.Meal'
    master_key = 'restaurant'
    column_names = 'person what'


class MealsByPerson(dd.Table):
    model = 'app.Meal'
    master_key = 'person'
    column_names = 'restaurant what'





from __future__ import unicode_literals
from lino.api import dd
from django.db import models
from django.core.exceptions import ValidationError



class Author(dd.Model):
    first_name = models.CharField("First name", max_length=50)
    last_name = models.CharField("Last name", max_length=50)
    country = models.CharField("Country", max_length=50, blank=True)
    
    def __str__(self):
        return "%s, %s" % (self.last_name, self.first_name)


class Book(dd.Model):
    author = dd.ForeignKey(Author, blank=True, null=True)
    title = models.CharField("Title", max_length=200)
    published = models.IntegerField(
        "Published",
        help_text="The year of publication")
    price = models.DecimalField("Price", decimal_places=2, max_digits=10)


    def full_clean(self):
        super(Book, self).full_clean()
        if self.published > 2000 and self.price < 5:
            price = dd.format_currency(self.price)
            msg = "A book from {} for only {}!".format(
                self.published, price)
            raise ValidationError(msg)
                

        



# Copyright 2010-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from __future__ import unicode_literals
from builtins import str
from lino.api import dd
from django.db import models
from lino.mixins.polymorphic import Polymorphic


class Person(models.Model):

    name = models.CharField(max_length=50)

    def __str__(self):
        return self.name



class Place(Polymorphic):

    name = models.CharField(max_length=50)
    owners = models.ManyToManyField(Person)

    def __str__(self):
        return "%s (owners=%s)" % (
            self.name,
            ', '.join([str(o) for o in self.owners.all()]))



class Restaurant(Place):

    serves_hot_dogs = models.BooleanField(default=False)
    cooks = models.ManyToManyField(Person)

    def __str__(self):
        return "%s (owners=%s, cooks=%s)" % (
            self.name,
            ', '.join([str(o) for o in self.owners.all()]),
            ', '.join([str(o) for o in self.cooks.all()]))



class Visit(models.Model):

    allow_cascaded_delete = ['place']
    person = dd.ForeignKey(Person)
    place = dd.ForeignKey(Place)
    purpose = models.CharField(max_length=50)

    def __str__(self):
        return "%s visit by %s at %s" % (
            self.purpose, self.person, self.place.name)



class Meal(models.Model):

    allow_cascaded_delete = ['restaurant']
    person = dd.ForeignKey(Person)
    restaurant = dd.ForeignKey(Restaurant)
    what = models.CharField(max_length=50)

    def __str__(self):
        return "%s eats %s at %s" % (
            self.person, self.what, self.restaurant.name)



from lino.api import ad, _


class Plugin(ad.Plugin):
    verbose_name = _("Virtual tables")

    def setup_main_menu(self, site, profile, m):
        m = m.add_menu(self.app_label, self.verbose_name)
        m.add_action('vtables.CitiesAndInhabitants')

    



from lino.api import dd


class Authors(dd.Table):
    model = 'Author'
    column_names = 'first_name last_name country'

    detail_layout = """
    first_name last_name country
    BooksByAuthor
    """


class Books(dd.Table):
    model = 'Book'
    column_names = 'author title published *'
    hide_sums = True


class RecentBooks(Books):
    column_names = 'published title author'
    order_by = ['published']


class BooksByAuthor(Books):
    master_key = 'author'
    column_names = 'published title'
    order_by = ['published']



# -*- coding: UTF-8 -*-
# Copyright 2016-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Fixtures specific for Lino Care.

.. autosummary::
   :toctree:

   models


"""

from lino_xl.lib.tickets import *
from lino.api import _


# class Plugin(Plugin):
#     verbose_name = _("Pleas")
#     extends_models = ['Ticket']
    



# -*- coding: UTF-8 -*-
# Copyright 2017-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Defines and instantiates a demo version of a Lino Avanti Site."""

import datetime

from ..settings import *


class Site(Site):

    the_demo_date = datetime.date(2017, 2, 15)
    languages = "en de fr"

    def get_plugin_configs(self):
        for i in super(Site, self).get_plugin_configs():
            yield i
        yield ('beid', 'simulate_eidreader_path', self.project_dir.child('simulate_eidreader'))

SITE = Site(globals())

DEBUG = True

# the following line should not be active in a checked-in version
#~ DATABASES['default']['NAME'] = ':memory:'

# SITE.eidreader_timeout = 25



from lino_book.projects.estref.settings import *
SITE = Site(globals(), title=Site.verbose_name + " demo")
#~ DEBUG=True
# the following line should always be commented out in a checked-in version
#~ DATABASES['default']['NAME'] = ':memory:'



# -*- coding: UTF-8 -*-
# Copyright 2015-2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""The :xfile:`settings.py` modules for this variant.

.. autosummary::
   :toctree:

   demo
   doctests

"""

from lino_book.projects.team.settings.demo import *


class Site(Site):

    default_ui = 'lino_noi.lib.public'
    default_user = 'anonymous'

    # def get_installed_apps(self):
    #     yield super(Site, self).get_installed_apps()
    #     yield 'lino.modlib.bootstrap3'




# -*- coding: UTF-8 -*-
# Copyright 2014-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""
Runs some tests about reading eID cards.

You can run only these tests by issuing::

  $ go avanti1
  $ python manage.py test tests.test_beid

"""

from __future__ import unicode_literals
from __future__ import print_function

import os
import json

from lino.utils.djangotest import RemoteAuthTestCase
from django.utils.datastructures import MultiValueDict
from lino.utils import ssin
from lino.api import dd, rt


def readfile(name):
    fn = os.path.join(os.path.dirname(__file__), name)
    return open(fn).read()


# class WebRequest:
#     method = "POST"
#     subst_user = None
#     requesting_panel = None

#     def __init__(self, user, data):
#         self.POST = self.REQUEST = MultiValueDict(data)
#         self.user = user


class BeIdTests(RemoteAuthTestCase):
    maxDiff = None
    # override_djangosite_settings = dict(use_java=True)

    def test01(self):
        from lino.core import constants
        from django.conf import settings
        from lino.modlib.users.choicelists import UserTypes
        from lino.api.shell import countries, avanti, users

        # is it the right settings module?
        self.assertEqual(os.environ['DJANGO_SETTINGS_MODULE'],
                         'lino_book.projects.avanti1.settings.demo')

        self.assertEqual(settings.MIDDLEWARE, (
            'django.middleware.common.CommonMiddleware',
            'django.middleware.locale.LocaleMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'lino.core.auth.middleware.AuthenticationMiddleware',
            'lino.core.auth.middleware.WithUserMiddleware',
            'lino.core.auth.middleware.DeviceTypeMiddleware',
            'lino.core.auth.middleware.RemoteUserMiddleware',
            'lino.utils.ajax.AjaxExceptionResponse'))

        u = users.User(username='robin',
                       user_type=UserTypes.admin,
                       language="en")
        u.save()
        self.client.force_login(u)
        be = countries.Country(name="Belgium", isocode="BE")
        be.save()

        Holder = dd.plugins.beid.holder_model
        kw = dict()
        # kw.update(card_number="123456789")
        # kw.update(national_id="680601 053-29")
        kw.update(first_name="Jean-Jacques")
        # kw.update(middle_name="")
        kw.update(last_name="Jeffin")
        obj = Holder(**kw)
        obj.full_clean()
        obj.save()

        def simulate_eidreader(uuid):
            # simulate the client's eidreader posting its data to the
            # server.
            s = readfile(uuid + '.json')
            # raise Exception(repr(s))
            data = dict(card_data=s)
            url = '/eid/' + uuid
            response = self.client.post(
                url, data,
                REMOTE_USER='robin',
                HTTP_ACCEPT_LANGUAGE='en')
            result = self.check_json_result(response, 'success message')
            self.assertEqual(result, {'message': 'OK', 'success': True})

        # The following tests are based on an older simulation system developed
        # before simulate_eidreader_path was developed. TODO: convert them to
        # simulate_eidreader_path. En attendant we simply disable
        # simulate_eidreader_path here. Works as well.
        save_path = dd.plugins.beid.simulate_eidreader_path
        dd.plugins.beid.simulate_eidreader_path = None
        
        uuid = 'beid_test_1'
        simulate_eidreader(uuid)

        url = '/api/avanti/Clients'
        post_data = dict()
        # post_data.update(
        #     card_data=readfile('beid_test_1.json'))
        post_data['uuid'] = uuid
        post_data[constants.URL_PARAM_ACTION_NAME] = 'find_by_beid'

        # First attempt fails because a person with exactly the same
        # name already exists.
        response = self.client.post(
            url, post_data,
            REMOTE_USER='robin',
            HTTP_ACCEPT_LANGUAGE='en')

        # result = self.check_json_result(response, 'alert success message')
        result = self.check_json_result(response)
        # raise Exception(repr(result))
        self.assertEqual(result['success'], False)
        expected = ("Sorry, I cannot handle that case: Cannot create "
                    "new client because there is already a person named "
                    "Jean-Jacques Jeffin in our database.")
        self.assertEqual(result['message'], expected)

        # Second attempt. We are reading the same card, but this time
        # there is a person with this `national_id`.
        obj.national_id = "680601 053-29"
        # obj.first_name = "Jean-Claude"
        obj.full_clean()
        obj.save()


        dlg = []
        expected = """\
Click OK to apply the following changes for JEFFIN Jean-Jacques (100) :\
<br/>Birth date : '' -> 1968-06-01
<br/>Birth place : '' -> 'Mons'
<br/>Country : None -> Country #BE ('Belgium')
<br/>Gender : None -> <Genders.male:M>
<br/>ID card valid from : None -> 2016-02-06
<br/>Locality : None -> Place #1 ('Helsinki')
<br/>Street : '' -> 'Estland'
<br/>Zip code : '' -> '1262'
<br/>eID card issuer : '' -> 'Helsinki'
<br/>eID card number : '' -> '592382784772'
<br/>eID card type : None -> <BeIdCardTypes.belgian_citizen:01>
<br/>until : None -> 2026-02-06"""

# Click OK to apply the following changes for JEFFIN Jean (100) :\
# <br/>Locality : None -> Place #1 ('Tallinn')
# <br/>Gender : None -> <Genders.male:M>
# <br/>until : None -> 2016-08-19
# <br/>Street : '' -> 'Estland'
# <br/>ID card valid from : None -> 2011-08-19
# <br/>eID card type : None -> <BeIdCardTypes.belgian_citizen:1>
# <br/>eID card issuer : '' -> 'Tallinn'
# <br/>Birth place : '' -> 'Mons'
# <br/>Country : None -> Country #BE ('Belgium')
# <br/>Birth date : '' -> 1968-06-01
# <br/>eID card number : '' -> '592345678901'
# <br/>Zip code : '' -> '1418'

        dlg.append((expected, 'yes'))
        dlg.append((
            'Client "JEFFIN Jean-Jacques (100)" has been saved.',
            None))
        self.check_callback_dialog(
            self.client.post, 'robin', url, dlg, post_data)

        obj = avanti.Client.objects.get(id=100)
        # addr = addresses.Address.objects.get(partner=obj)
        self.assertEqual(obj.city.name, "Helsinki")
        # self.assertEqual(addr.primary, True)

        # Third attempt. A person with almost same name and same
        # national_id.

        obj.national_id = "680601 053-29"
        obj.first_name = "Jean"
        obj.middle_name = "Jacques"
        obj.full_clean()
        obj.save()
        self.assertEqual(obj.national_id, "680601 053-29")
        response = self.client.post(
            url, post_data,
            REMOTE_USER='robin',
            HTTP_ACCEPT_LANGUAGE='en')
        result = self.check_json_result(
            response,
            'xcallback success message')
        self.assertEqual(result['success'], True)
        expected = """\
Click OK to apply the following changes for JEFFIN Jean-Jacques (100) :<br/>First name : 'Jean' -> 'Jean-Jacques'
<br/>Middle name : 'Jacques' -> ''"""
        # print(result['message'])
        self.assertEqual(result['message'], expected)


        # Fourth attempt. A person with slightly different name and
        # equivalent but wrongly formatted national_id exists.  Lino
        # does not recognize this duplicate here. To avoid this case,
        # the StrangeClients table warns about wrongly formatted
        # national_id fields.

        Holder.validate_national_id = False

        ssin.parse_ssin('68060105329')
        obj.national_id = "68060105329"
        obj.first_name = "Jean Jacques"
        obj.middle_name = ""
        # obj.client_state = ClientStates.coached
        # obj.update_dupable_words()  # avoid repairable message
        obj.full_clean()
        obj.save()
        self.assertEqual(obj.national_id, "68060105329")
        response = self.client.post(
            url, post_data,
            REMOTE_USER='robin',
            HTTP_ACCEPT_LANGUAGE='en')
        # self.assertEqual(response.content, '')
        result = self.check_json_result(
            response,
            'xcallback success message')
        self.assertEqual(result['success'], True)
        expected = "Create new client Jean-Jacques Jeffin : Are you sure?"
        # print(result['message'])
        self.assertEqual(result['message'], expected)

        # test whether we would have been warned:
        obj.update_dupable_words()  # avoid repairable message
        ar = rt.models.checkdata.ProblemsByOwner.request(
            master_instance=obj)
        obj.check_data(fix=False)
        s = ar.to_rst()
        # print(s)
        self.assertEqual(s, """\
*(★) Malformed SSIN '68060105329' must be '680601 053-29'.*
""")

        obj.check_data(fix=True)
        ar = rt.models.checkdata.ProblemsByOwner.request(
            master_instance=obj)
        s = ar.to_rst()
        # print(s)
        self.assertEqual(s, "\n")

        # Last attempt for this card. No similar person exists. Create
        # new client from eid.

        obj.first_name = "Jean-Claude"
        obj.national_id = ""
        obj.full_clean()
        obj.save()
        response = self.client.post(
            url, post_data,
            REMOTE_USER='robin',
            HTTP_ACCEPT_LANGUAGE='en')
        # self.assertEqual(response.content, '')
        result = self.check_json_result(
            response,
            'xcallback success message')
        self.assertEqual(result['success'], True)
        expected = "Create new client Jean-Jacques Jeffin : Are you sure?"
        self.assertEqual(result['message'], expected)

        dd.plugins.beid.eidreader_timeout = 1

        # when eidreader is not installed on client, there will be no
        # incoming POST and therefore we will have a timeout.
        post_data['uuid'] = 'foo'  #
        response = self.client.post(
            url, post_data,
            REMOTE_USER='robin',
            HTTP_ACCEPT_LANGUAGE='en')
        # self.assertEqual(response.content, '')
        result = self.check_json_result(
            response,
            'alert success message')
        self.assertEqual(result['success'], False)
        self.assertEqual(result['message'], "Abandoned after 1 seconds")

        dd.plugins.beid.eidreader_timeout = 15

        uuid = 'beid_test_0'
        simulate_eidreader(uuid)

        post_data['uuid'] = uuid
        response = self.client.post(
            url, post_data,
            REMOTE_USER='robin',
            HTTP_ACCEPT_LANGUAGE='en')
        # self.assertEqual(response.content, '')
        result = self.check_json_result(
            response,
            'alert success message')
        self.assertEqual(result['success'], False)
        self.assertEqual(result['message'], "No card data found: Could not find any reader with a card inserted")



        if True:
            # skip the following tests because we don't yet have
            # test data for the Python eidreader.
            return

        # next card. a foreigner card with incomplete birth date

        post_data.update(card_data=readfile('beid_tests_2.txt'))
        response = self.client.post(
            url, post_data,
            REMOTE_USER='robin',
            HTTP_ACCEPT_LANGUAGE='en')
        # self.assertEqual(response.content, '')
        result = self.check_json_result(
            response,
            'xcallback success message')
        self.assertEqual(result['success'], True)
        expected = "Create new client Marc Petitjean : Are you sure?"
        self.assertEqual(result['message'], expected)

        # next card. issued after 2015 and the photo is invalid

        post_data.update(card_data=readfile('beid_tests_3.txt'))
        response = self.client.post(
            url, post_data,
            REMOTE_USER='robin',
            HTTP_ACCEPT_LANGUAGE='en')
        # self.assertEqual(response.content, '')
        result = self.check_json_result(
            response,
            'xcallback success message')
        self.assertEqual(result['success'], True)
        expected = "Create new client Bernd Brecht : Are you sure?"
        self.assertEqual(result['message'], expected)

        # next card. issued after 2015 and the photo is valid

        post_data.update(card_data=readfile('beid_tests_4.txt'))
        response = self.client.post(
            url, post_data,
            REMOTE_USER='robin',
            HTTP_ACCEPT_LANGUAGE='en')
        # self.assertEqual(response.content, '')
        result = self.check_json_result(
            response,
            'xcallback success message')
        self.assertEqual(result['success'], True)
        expected = "Create new client Jean Dupont : Are you sure?"
        self.assertEqual(result['message'], expected)

        dd.plugins.beid.simulate_eidreader_path = save_path



# -*- coding: UTF-8 -*-
# Copyright 2015-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

# go gfktest
# python manage.py test

from __future__ import unicode_literals
from builtins import str
# from lino.utils.test import DocTest
import six

from lino.utils.djangotest import WebIndexTestCase

from django.db import models
from django.conf import settings

from lino.api import rt
from lino.utils.djangotest import TestCase


class TestCase(TestCase):

    maxDiff = None

    def test01(self):
        """We create a member, and three GFK-related objects whose `owner`
        fields point to that member. And then we try to delete that
        member.

        """
        Member = rt.models.gfktest.Member
        Note = rt.models.gfktest.Note
        Memo = rt.models.gfktest.Memo
        Comment = rt.models.gfktest.Comment
        BrokenGFKs = rt.models.gfks.BrokenGFKs

        def check_status(*args):
            for i, m in enumerate((Member, Comment, Note, Memo)):
                n = m.objects.all().count()
                if n != args[i]:
                    msg = "Expected %d objects in %s but found %d"
                    msg %= (args[i], m.__name__, n)
                    self.fail(msg)
        
        gfklist = [
            (f.model, f.fk_field, f.ct_field)
            for f in settings.SITE.kernel.GFK_LIST]
        self.assertEqual(gfklist, [
            (Comment, 'owner_id', 'owner_type'),
            (Memo, 'owner_id', 'owner_type'),
            (Note, 'owner_id', 'owner_type')])

        def create_objects():
            mbr = Member(name="John",id=1)
            mbr.save()

            self.assertEqual(mbr.name, "John")
            Comment(owner=mbr, text="Just a comment...").save()
            Note(owner=mbr, text="John owes us 100€").save()
            Memo(owner=mbr, text="More about John and his friends").save()
            return mbr

        mbr = create_objects()
        check_status(1, 1, 1, 1)
        try:
            mbr.delete()
        except Warning as e:
            self.assertEqual(
                str(e), "Cannot delete member John because 1 notes refer to it.")
        else:
            self.fail("Expected an exception")

        # they are all still there:
        check_status(1, 1, 1, 1)
        
        # delete the note manually
        Note.objects.all().delete()
        check_status(1, 1, 0, 1)
        mbr.delete()
        # the memo remains:
        check_status(0, 0, 0, 1)
        Memo.objects.all().delete()

        # The above behaviour is thanks to a `pre_delete_handler`
        # which Lino adds automatically. Theoretically it is no longer
        # possible to produce broken GFKs.  But now we disable this
        # `pre_delete_handler` and use Django's raw `delete` method in
        # order to produce some broken GFKs:

        from django.db.models.signals import pre_delete
        from lino.core.model import pre_delete_handler
        pre_delete.disconnect(pre_delete_handler)

        check_status(0, 0, 0, 0)
        mbr = create_objects()
        check_status(1, 1, 1, 1)
        models.Model.delete(mbr)

        pre_delete.connect(pre_delete_handler)

        # The member has been deleted, but all generic related objects
        # are still there:
        check_status(0, 1, 1, 1)

        # That's what the BrokenGFKs table is supposed to show:
        # rst = BrokenGFKs.request().table2rst()
        rst = BrokenGFKs.request().to_rst()
        self.assertEqual(rst, """\
====================== ================== ======================================================== ========
 Database model         Database object    Message                                                  Action
---------------------- ------------------ -------------------------------------------------------- --------
 `comment <Detail>`__   *Comment object*   Invalid primary key 1 for gfktest.Member in `owner_id`   delete
 `note <Detail>`__      *Note object*      Invalid primary key 1 for gfktest.Member in `owner_id`   manual
 `memo <Detail>`__      *Memo object*      Invalid primary key 1 for gfktest.Member in `owner_id`   clear
====================== ================== ======================================================== ========

""")




from unittest import TestCase
from django import VERSION
from lino_book.projects.diamond.main.models import PizzeriaBar

# install the fix
from lino.core.inject import django_patch
django_patch()


class DocTest(TestCase):

    def test_django(self):
        p = PizzeriaBar(name="Michaels", min_age=21, specialty="Cheese",
                        pizza_bar_specific_field="Doodle")
        self.assertEqual(p.pizza_bar_specific_field, 'Doodle')
        
        if VERSION[0] == 1 and VERSION[1] == 6:
            self.check_django_16(p)
        else:
            self.check_django_17(p)

    def check_django_16(self, p):
        
        self.assertEqual(p.name, '')

        # The `name` field has not been initialized because
        # it is being inherited from a grand-parent.
        
    def check_django_17(self, p):
        self.assertEqual(p.name, 'Michaels')
        
        



# -*- coding: utf-8 -*-
# Copyright 2014-2015 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Test some things which happen when a database object is deleted.

This module is part of the Lino test suite. You can test only this
module by issuing either::

  $ go min2
  $ python manage.py test
  $ python manage.py test tests.test_min2.QuickTest.test_dupable

or::

  $ go book
  $ python setup.py test -s tests.ProjectsTests.test_min2


When I delete a database object, Lino also deletes those objects who
are related through a GenericForeignKey.

For example excerpts (`excerpts.Excerpt`) are related to their
"controller" or "owner" via a *generic* foreign key (as every
Controllable).  What happens to an excerpt when you delete its owner?
The default behaviour is to delete them silently (cascaded delete).

Another example are notes (`notes.Note`) who are also Controllable.
But unlike excerpts, we don't want them to vanish when we delete their
owner. We want Lino to tell us "Cannot delete this record because
other database objects are referring to it".

"""

from __future__ import unicode_literals
from __future__ import print_function

from lino.api import rt

from lino.modlib.gfks.mixins import Controllable

from lino.utils.djangotest import RemoteAuthTestCase

from lino.core.utils import full_model_name


def create(m, **kw):
    obj = m(**kw)
    obj.full_clean()
    obj.save()
    obj.after_ui_save(None, None)
    return obj


class QuickTest(RemoteAuthTestCase):

    fixtures = ['std', 'few_countries']

    def test_controllables(self):
        found = []
        for M in rt.models_by_base(Controllable):
            found.append(full_model_name(M))
        expected = """cal.Event cal.Task
        comments.Comment
        excerpts.Excerpt
        notes.Note
        notify.Message
        checkdata.Problem
        """.split()
        self.assertEqual(found, expected)

        Person = rt.models.contacts.Person
        Note = rt.models.notes.Note
        Excerpt = rt.models.excerpts.Excerpt
        ExcerptType = rt.models.excerpts.ExcerptType
        ContentType = rt.models.contenttypes.ContentType
        self.assertEqual(len(Person.allow_cascaded_delete), 0)
        self.assertEqual(len(Note.allow_cascaded_delete), 0)
        self.assertEqual(len(Excerpt.allow_cascaded_delete), 1)

        doe = create(Person, first_name="John", last_name="Doe")
        note = create(Note, owner=doe, body="John Doe is a fink!")
        ct = ContentType.objects.get_for_model(Note)
        etype = create(ExcerptType, name="Note", content_type=ct)
        excerpt = create(Excerpt, owner=note,
                         excerpt_type=etype)

        self.assertEqual(Person.objects.count(), 1)
        self.assertEqual(Note.objects.count(), 1)
        self.assertEqual(Excerpt.objects.count(), 1)

        ar = rt.models.notes.Notes.request()
        s = ar.to_rst(column_names="id owner")
        self.assertEqual(s, """\
==== ===============
 ID   Controlled by
---- ---------------
 1    *John Doe*
==== ===============

""")

        ar = rt.models.excerpts.Excerpts.request()
        s = ar.to_rst(column_names="id owner")
        self.assertEqual(s, """\
==== ===============
 ID   Controlled by
---- ---------------
 1    *Note #1*
==== ===============

""")

        self.assertEqual(excerpt.disable_delete(), None)
        self.assertEqual(note.disable_delete(), None)
        self.assertEqual(doe.disable_delete(), None)
        # it is not "Cannot delete John Doe because 1 Notes refer to
        # it." because Note.owner is nullable.

        note.delete()
        self.assertEqual(Excerpt.objects.count(), 0)
        self.assertEqual(ExcerptType.objects.count(), 1)

    def test_dupable(self):
        
        Company = rt.models.contacts.Company
        Person = rt.models.contacts.Person
        DupableWord = rt.models.dupable_partners.Word

        bernard = create(Person, first_name="Bernard", last_name="Bodard")
        
        self.assertEqual(Person.objects.count(), 1)
        self.assertEqual(bernard.dupable_words.count(), 2)
        self.assertEqual(DupableWord.objects.count(), 2)
        words = [o.word for o in DupableWord.objects.filter(owner=bernard)]
        self.assertEqual(words, [u'PTRT', u'PRNRT'])

        bernard2 = create(Person, first_name="Bodard", last_name="Bernard")
        bernard3 = create(
            Person, first_name="Bernhard", last_name="Bodard")
        godard = create(Person, first_name="Bernard", last_name="Godard")
        erna = create(Person, first_name="Erna", last_name="Odar")
        bernard4 = create(
            Person, first_name="Bernard-Marie", last_name="Bodard")
        marie = create(Person, first_name="Marie", last_name="Bernard-Bodard")

        create(Company, name="Külamaja OÜ")

        def create_person(first, last):
            return create(Person, first_name=first, last_name=last)

        mm1 = create_person("Marie", "Meier")
        mm2 = create_person("Marie", "Meyer")

        def check(obj, expected):
            got = map(str, obj.find_similar_instances(4))
            got = '\n'.join(got)
            self.assertEqual(got, expected.strip())

        check(bernard, """
Bodard Bernard
Bernhard Bodard
Bernard-Marie Bodard
Marie Bernard-Bodard
""")

        check(bernard2, """
Bernard Bodard
Bernhard Bodard
Bernard-Marie Bodard
Marie Bernard-Bodard
""")

        check(bernard3, """
Bernard Bodard
Bodard Bernard
Bernard-Marie Bodard
Marie Bernard-Bodard
""")

        check(godard, "")
        check(erna, "")
        check(bernard4, """
Marie Bernard-Bodard""")
        check(marie, """
Bernard-Marie Bodard
Bernard Bodard
Bodard Bernard
Bernhard Bodard
""")

        check(mm1, "Marie Meyer")
        check(mm2, "Marie Meier")



from django.db import models

class A(models.Model):
    price = models.DecimalField(max_digits=10, decimal_places=2)
    qty = models.DecimalField(max_digits=5, decimal_places=2)
    
    def total(self):
        return self.price * self.qty





# -*- coding: UTF-8 -*-
# Copyright 2015-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from __future__ import unicode_literals
from __future__ import print_function

from lino.api import rt, dd, _
from lino.utils import Cycler

from lino.utils.mldbc import babel_named as named

def skills_objects():
    "was previously in skills.fixtures.demo2"

    Skill = rt.models.skills.Skill
    Competence = rt.models.skills.Competence
    Demand = rt.models.skills.Demand
    # Ticket = rt.models.tickets.Ticket
    User = rt.models.users.User

    yield named(Skill, _('Psychotherapy'))
    yield named(Skill, _('Psychiatry'))

    SKILLS = Cycler(Skill.objects.all())
    END_USERS = Cycler(dd.plugins.skills.end_user_model.objects.all())

    i = 0
    for j in range(2):
        for u in User.objects.all():
            i += 1
            yield Competence(user=u, faculty=SKILLS.pop())
            if i % 2:
                yield Competence(user=u, faculty=SKILLS.pop())
            if i % 3:
                yield Competence(
                    user=u, faculty=SKILLS.pop(),
                    end_user=END_USERS.pop())
            
    for i, t in enumerate(
            dd.plugins.skills.demander_model.objects.all()):
        yield Demand(demander=t, skill=SKILLS.pop())
        if i % 3:
            yield Demand(demander=t, skill=SKILLS.pop())
            

def objects():
    return []



# -*- coding: UTF-8 -*-
# Copyright 2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Some demo data for the Vilma project.

"""

from lino.api import dd, rt, _
from lino.utils.mldbc import babel_named as named
from lino.utils.cycler import Cycler
from lino_xl.lib.cal.choicelists import Recurrencies

def objects():
    # not used. from the time when we still had tickets and courses
    Line = rt.models.courses.Line
    Topic = rt.models.courses.Topic
    Course = rt.models.courses.Course
    EventType = rt.models.cal.EventType
    Room = rt.models.cal.Room
    Company = rt.models.contacts.Company
    Person = rt.models.contacts.Person
    Role = rt.models.contacts.Role
    RoleType = rt.models.contacts.RoleType

    school = named(Room, _("School"))
    yield school
    center = named(Room, _("Youth center"))
    yield center
    library = named(Room, _("Library"))
    yield library
    
    training = named(EventType, _("Training"))
    yield training
    workshop = named(EventType, _("Workshop"))
    yield workshop
    camp = named(EventType, _("Camp"))
    yield camp
    
    nature = named(Topic, _("Nature"))
    yield nature
    folk = named(Topic, _("Folk"))
    yield folk
    together = named(Topic, _("Acting together"))
    yield together
    health = named(Topic, _("Health"))
    yield health
    comp = named(Topic, _("Computer"))
    yield comp
    
    yield named(
        Line, _("Photography workshop"),
        event_type=workshop,
        every_unit=Recurrencies.once, topic=together)
    yield named(
        Line, _("Teamwork training"),
        event_type=training,
        every_unit=Recurrencies.once, topic=together)
    yield named(
        Line, _("Folk camp 2017"),
        event_type=camp,
        every_unit=Recurrencies.once, topic=together)
    yield named(
        Line, _("Lino Vilma training"),
        event_type=training,
        every_unit=Recurrencies.weekly, topic=together)

    LINES = Cycler(Line.objects.all())
    for offset in (-60, -10, -5, 1, 10, 30):
        yield Course(line=LINES.pop(), start_date=dd.demo_date(offset))

    choir = Company(name="Village choir")
    yield choir
    
    yield Company(name="Sopranos", parent=choir)
    yield Company(name="Altos", parent=choir)
    yield Company(name="Tenors", parent=choir)
    yield Company(name="Basses", parent=choir)
    
    RTYPES = Cycler(RoleType.objects.all())
    COMPANIES = Cycler(Company.objects.all())

    for i, p in enumerate(Person.objects.all()):
        for j in range(i % 3):
            yield Role(company=COMPANIES.pop(), type=RTYPES.pop(),
                       person=p)
        



@dd.chooser()
def food_choices(cls, ar):
    year_in_school = ar.get_user().year_in_school if ar is not None else None
    food = []
    for name, reserved_for in MENU:
        if (year_in_school is None) or (reserved_for is None) or year_in_school in reserved_for:
            food.append(name)
    return food



from django.db import models
from lino.api import dd, _

from lino_book.projects.chooser.food import year_in_school, food_choices, food



class Country(dd.Model):
    class Meta(object):
        verbose_name = _("Country")
        verbose_name_plural = _("Countries")

    name = models.CharField(max_length=20)

    def __str__(self):
        return self.name



class City(dd.Model):
    class Meta(object):
        verbose_name = _("City")
        verbose_name_plural = _("Cities")

    name = models.CharField(max_length=20)
    country = dd.ForeignKey(Country)

    def __str__(self):
        return self.name



class Contact(dd.Model):
    class Meta(object):
        verbose_name = _("Contact")
        verbose_name_plural = _("Contacts")

    name = models.CharField(max_length=20)
    country = dd.ForeignKey(Country, blank=True, null=True)
    city = dd.ForeignKey(City, blank=True, null=True)
    year_in_school = year_in_school
    food = food

    def __str__(self):
        return self.name

    @dd.chooser()
    def city_choices(cls, country):
        if country is not None:
            return country.city_set.order_by('name')
        return cls.city.field.remote_field.model.objects.order_by('name')

    food_choices = food_choices



class Contacts(dd.Table):
    model = Contact


class Countries(dd.Table):
    model = Country
    detail_layout = """name id
    CitiesByCountry
    """

class Cities(dd.Table):
    model = City
    detail_layout = """name country id
    ContactsByCity
    """


class CitiesByCountry(Cities):
    master_key = "country"


class ContactsByCity(Contacts):
    master_key = "city"
    column_names = "name year_in_school food *"



from lino.api import dd


class Tag(dd.Model):

    name = dd.CharField(max_length=100)

    def __str__(self):
        return self.name

@dd.receiver(dd.auto_create)
def my_auto_create_handler(sender, **kw):
    print("My handler was called with {}".format(sender))



"""This package contains example applications included with the Lino
book.  These can be used either out of the box or as a base for
derivated work.

See :doc:`/dev/projects` which explains how to try them.

Usage examples of real-world :term:`Lino applications <Lino application>`:


.. autosummary::
   :toctree:

   apc
   pierre
   cosi_ee
   team
   bs3
   public
   anna
   liina
   lydia
   avanti1

Examples used by a tutorial:

.. autosummary::
   :toctree:

   chooser
   min1
   mti
   nomti
   lets1
   lets2
   actors
   actions
   tables
   dumps
   combo
   polly
   polls
   polls2
   watch
   watch2
   workflows


Poorly documented examples at different development states:

.. autosummary::
   :toctree:

   min2
   chatter
   max
   cms
   belref
   estref


Technical

.. autosummary::
   :toctree:

   docs

Sleeping:

.. autosummary::
   :toctree:

   events
   crl
   homeworkschool
   igen
   i18n

"""



# -*- coding: UTF-8 -*-
# Copyright 2014-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)


"""
Default settings for a :ref:`cosi` site "à la Pierre".

"""

from __future__ import unicode_literals

from lino_cosi.lib.cosi.settings import *


class Site(Site):
    languages = 'fr en'
    # demo_fixtures = 'std few_countries minimal_ledger euvatrates \
    # furniture demo demo_bookings demo2'.split()
    demo_fixtures = 'std few_countries minimal_ledger \
    furniture demo demo_bookings demo2'.split()

    #def get_installed_apps(self):
    #    yield super(Site, self).get_installed_apps()
    #    yield 'lino_xl.lib.bevat'

    def setup_plugins(self):
        self.plugins.countries.configure(hide_region=False)
        self.plugins.ledger.configure(use_pcmn=True)
        self.plugins.countries.configure(country_code='BE')
        super(Site, self).setup_plugins()




from lino.projects.std.settings import *


class Site(Site):

    title = "Cool Polls"

    demo_fixtures = "demo demo2"

    project_name = "pools2_mysite"  # avoid name clash when LINO_CACHE_ROOT is set

    def get_installed_apps(self):
        yield 'polls'
        yield 'lino.modlib.users'
        yield super(Site, self).get_installed_apps()

    def setup_menu(self, user_type, main):
        super(Site, self).setup_menu(user_type, main)
        m = main.add_menu("polls", "Polls")
        m.add_action('polls.Questions')
        m.add_action('polls.Choices')

SITE = Site(globals())

# your local settings here

DEBUG = True



# import os ; print "20161219 %s (pid:%s)" % (__name__, os.getpid())

import datetime

from ..settings import *


class Site(Site):
    """Defines and instantiates a demo version of Lino Care."""

    # default_ui = 'lino_extjs6.extjs6'
    
    the_demo_date = datetime.date(2015, 5, 23)

    languages = "en de fr"

    def unused_setup_plugins(self):
        """Change the default value of certain plugin settings.

        - :attr:`excerpts.responsible_user
          <lino_xl.lib.excerpts.Plugin.responsible_user>` is set to
          ``'jean'`` who is both senior developer and site admin in
          the demo database.

        """
        super(Site, self).setup_plugins()
        if self.is_installed('excerpts'):
            self.plugins.excerpts.configure(responsible_user='robin')

            # print "20161219 a"
SITE = Site(globals())
# print "20161219 b"
DEBUG = True

# the following line should not be active in a checked-in version
# DATABASES['default']['NAME'] = ':memory:'



from lino.projects.std.settings import *


SITE = Site(globals(), 'lino_book.projects.20100212')

# INSTALLED_APPS = (
#     'lino',
#     'lino_book.projects.20100212',  # Explaining Django ticket 12801
# )

#~ LOGGING = dict(filename='system.log',level='DEBUG',mode='w')



# -*- coding: UTF-8 -*-
# Copyright 2017-2020 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from lino.projects.std.settings import *

from lino.api import _

class Site(Site):
    verbose_name = "Lino Chatter"
    description = _("A simple chatting app")

    demo_fixtures = 'std demo demo2'.split()
    user_types_module = 'lino_xl.lib.xl.user_types'
    use_websockets = True

    languages = 'en'

    default_ui = "lino_react.react"

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.users'
        yield 'lino.modlib.notify'
        yield 'lino_xl.lib.groups'



from lino.projects.std.settings import *

class Site(Site):

    languages = 'en de de-be'

    demo_fixtures = ['demo']

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino_book.projects.de_BE'

    def setup_menu(self, user_type, main):
        m = main.add_menu("master", "Master")
        m.add_action('de_BE.Expressions')
        super(Site, self).setup_menu(user_type, main)
    

SITE = Site(globals())



from django.db import models
from lino.api import dd
from lino.utils import join_elems
from etgen.html import E
from lino.core.actors import qs2summary



class Place(dd.Model):
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name



class Member(dd.Model):
    name = models.CharField(max_length=200)
    place = dd.ForeignKey(Place, blank=True, null=True)
    email = models.EmailField(max_length=200, blank=True)

    def __str__(self):
        return self.name



class Product(dd.Model):
    name = models.CharField(max_length=200)

    providers = models.ManyToManyField(
        'lets.Member', verbose_name="Offered by",
        through='lets.Offer', related_name='offered_products')
    customers = models.ManyToManyField(
        'lets.Member', verbose_name="Wanted by",
        through='lets.Demand', related_name='wanted_products')

    def __str__(self):
        return self.name

    @dd.displayfield("Offered by")
    def offered_by(self, ar):
        if ar is None:
            return ''
        return qs2summary(ar, self.providers.all())

    @dd.displayfield("Wanted by")
    def wanted_by(self, ar):
        if ar is None:
            return ''
        return qs2summary(ar, self.customers.all())



class Offer(dd.Model):
    provider = dd.ForeignKey(Member)
    product = dd.ForeignKey(Product)
    valid_until = models.DateField(blank=True, null=True)

    def __str__(self):
        return "%s offered by %s" % (self.product, self.provider)



class Demand(dd.Model):
    customer = dd.ForeignKey(Member)
    product = dd.ForeignKey(Product)

    def __str__(self):
        return "%s (%s)" % (self.product, self.customer)





# -*- coding: UTF-8 -*-
# Copyright 2015-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""A :ref:`care` site with languages "en fr de".

.. autosummary::
   :toctree:

   lib
   user_types
   settings

"""




from unittest import TestCase
from django import VERSION
from .main.models import PizzeriaBar

# under Django 1.11 this fails with
# django.core.exceptions.FieldError: Local field u'street' in class 'PizzeriaBar' clashes with field of the same name from base class 'Pizzeria'.

class DocTest(TestCase):
    
    def test_django(self):
        p = PizzeriaBar(name="Michaels", min_age=21, specialty="Cheese",
                        pizza_bar_specific_field="Doodle")
        self.assertEqual(p.pizza_bar_specific_field, 'Doodle')
        
        if VERSION[0] == 1 and VERSION[1] == 6:
            self.check_django_16(p)
        elif VERSION[0] == 1 and VERSION[1] > 6:
            self.check_django_17(p)
        else:
            self.fail("Unsupported Django version {0}".format(VERSION))

    def check_django_16(self, p):
        
        self.assertEqual(p.name, '')

        # The `name` field has not been initialized because
        # it is being inherited from a grand-parent.
        
    def check_django_17(self, p):
        self.assertEqual(p.name, 'Michaels')
        
        



#!/usr/bin/env python
import os
import sys

if __name__ == "__main__":

    import os
    from django.core.management import execute_from_command_line

    os.environ["DJANGO_SETTINGS_MODULE"] = "lino_book.projects.watch.settings"
    execute_from_command_line(sys.argv)



# from lino.projects.std.settings import *
# SITE = Site(globals(), 'nomti')

from lino.projects.std.settings import *


class Site(Site):
    demo_fixtures = ['demo']
    def get_installed_apps(self):
        # yield 'lino.modlib.users'
        yield super(Site, self).get_installed_apps()
        yield 'lino_book.projects.nomti.app'

    def setup_menu(self, profile, main):
        m = main.add_menu("contacts", "Contacts")
        m.add_action('app.Persons')
        m.add_action('app.Places')
        m.add_action('app.Restaurants')

SITE = Site(globals())

DEBUG = True



# -*- coding: UTF-8 -*-
# Copyright 2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""
This is the base for all settings of lino_book.projects.dumps

.. autosummary::
   :toctree:

   demo


"""

from lino.projects.std.settings import *


class Site(Site):
    title = "Lino Dumps"

    demo_fixtures = 'demo'

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()

        yield 'lino_book.projects.dumps'



# -*- coding: utf-8 -*-
# Copyright 2014-2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Test certain aspects of `lino_xl.lib.addresses`.

This module is part of the Lino test suite. You can test only this
module by issuing either::

  $ go min2
  $ python manage.py test tests.test_addresses

or::

  $ go lino
  $ python setup.py test -s tests.ProjectsTests.test_min2


"""

from __future__ import unicode_literals
from __future__ import print_function

import logging
logger = logging.getLogger(__name__)

from lino.core.gfks import gfk2lookup
from lino.api import rt

from lino.modlib.gfks.mixins import Controllable

from lino.utils.djangotest import RemoteAuthTestCase

from lino.core.utils import full_model_name


def create(m, **kw):
    obj = m(**kw)
    obj.full_clean()
    obj.save()
    obj.after_ui_save(None, None)
    return obj


class QuickTest(RemoteAuthTestCase):

    fixtures = ['std', 'demo_users', 'few_countries', 'few_cities']

    def test_this(self):

        Company = rt.models.contacts.Company
        Address = rt.models.addresses.Address
        Place = rt.models.countries.Place
        Problem = rt.models.checkdata.Problem
        eupen = Place.objects.get(name="Eupen")
        ar = rt.models.contacts.Companies.request()
        self.assertEqual(Address.ADDRESS_FIELDS, set([
            'city', 'street_box', 'region', 'street_no',
            'street', 'addr2', 'addr1', 'country', 'zip_code']))
        
        def assert_check(obj, expected):
            qs = Problem.objects.filter(**gfk2lookup(Problem.owner, obj))
            got = '\n'.join([p.message for p in qs])
            self.assertEqual(got, expected)

        obj = create(Company, name="Owner with empty address")
        obj.check_data(ar, fix=False)
        assert_check(obj, '')
        obj.delete()

        self.assertEqual(Company.objects.count(), 0)
        self.assertEqual(Address.objects.count(), 0)

        doe = create(Company, name="Owner with address", city=eupen)

        self.assertEqual(Company.objects.count(), 1)
        self.assertEqual(Address.objects.count(), 0)

        assert_check(doe, '')  # No problems yet since not checked
        doe.check_data(ar, fix=False)
        assert_check(
            doe, '(\u2605) Owner with address, but no address record.')

        addr = doe.get_primary_address()
        self.assertEqual(addr, None)

        doe.check_data(ar, fix=True)
        assert_check(doe, '')  # problem has been fixed
        addr = doe.get_primary_address()
        self.assertEqual(Address.objects.count(), 1)
        self.assertEqual(addr.city, eupen)
        self.assertEqual(addr.primary, True)

        addr.primary = False
        addr.save()
        addr = doe.get_primary_address()
        self.assertEqual(addr, None)
        self.assertEqual(Address.objects.count(), 1)

        doe.check_data(ar, fix=False)
        assert_check(doe, '(\u2605) Unique address is not marked primary.')

        Address.objects.all().delete()
        self.assertEqual(Address.objects.count(), 0)
        addr = doe.get_primary_address()
        self.assertEqual(addr, None)

        doe.check_data(ar, fix=False)
        assert_check(
            doe, '(\u2605) Owner with address, but no address record.')

        doe.check_data(ar, fix=True)
        assert_check(doe, '')  # problem has been fixed

        # next problem : owner differs from primary address
        doe.city = None
        doe.zip_code = ''
        doe.full_clean()
        self.assertEqual(doe.city, None)
        doe.save()
        doe.check_data(ar, fix=False)
        self.assertEqual(Address.objects.count(), 1)
        assert_check(
            doe, "Primary address differs from owner address "
            "(city:Eupen->None, zip_code:4700->).")
        # Lino does repair this automatically since we don't know
        # which data is correct.
        doe.check_data(ar, fix=True)
        self.assertEqual(Address.objects.count(), 1)
        self.assertEqual(doe.city, None)
        addr = doe.get_primary_address()
        self.assertEqual(addr.city, eupen)
        self.assertEqual(addr.primary, True)
        
        # next problem: multiple primary address.
        # recover from previous test.
        doe.city = eupen
        doe.full_clean()
        doe.save()
        self.assertEqual(doe.city, eupen)
        self.assertEqual(doe.zip_code, '4700')
        addr = doe.get_primary_address()
        addr.id = None
        addr.save()
        self.assertEqual(Address.objects.count(), 2)
        doe.check_data(ar, fix=False)
        assert_check(doe, "Multiple primary addresses.")



import datetime

from ..settings import *


class Site(Site):
    # project_name = 'cosi_et'
    is_demo_site = True
    # ignore_dates_after = datetime.date(2019, 05, 22)
    the_demo_date = datetime.date(2019, 6, 12)
    # default_ui = 'lino_react.react'

    def get_plugin_configs(self):
        yield super(Site, self).get_plugin_configs()
        yield ('ledger', 'start_year', 2018)

SITE = Site(globals())
DEBUG = True



## Copyright 2013-2018 Rumma & Ko Ltd
## This file is part of the Lino project.

from lino.api import rt
from django.conf import settings

from lino.utils import Cycler

Entry = rt.models.watch2.Entry
EntryType = rt.models.watch2.EntryType
Company = rt.models.watch2.Company
User = rt.models.users.User

LOREM_IPSUM = "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."

def objects():

    kwargs = dict(user_type=rt.models.users.UserTypes.user)
    # kwargs = dict()
    
    yield User(username="Albert", **kwargs)
    yield User(username="Boris", **kwargs)
    yield User(username="Claire", **kwargs)
    
    yield Company(name="AllTech inc.")
    yield Company(name="BestTech inc.")
    yield Company(name="CoolTech inc.")
    
    yield EntryType(designation="Consultation")
    yield EntryType(designation="Evaluation")
    yield EntryType(designation="Test")
    yield EntryType(designation="Analysis")
    yield EntryType(designation="Observation")
    
    TYPES = Cycler(EntryType.objects.all())
    COMPANIES = Cycler(Company.objects.all())
    USERS = Cycler(User.objects.all())
    SUBJECTS = Cycler(LOREM_IPSUM.split())
    
    for i in range(200):
        d = settings.SITE.demo_date(-i)
        e = Entry(date=d,
            company=COMPANIES.pop(),
            user=USERS.pop(),
            subject=SUBJECTS.pop(),
            entry_type=TYPES.pop())
        if i % 7:
            yield e
    



from lino.utils.instantiator import Instantiator


def objects():
    b = Instantiator('addrloc.Company',
                     'name country zip_code city:name street street_no')
    yield b("First", "BE", "4700", "Eupen", "Favrunpark", 13)
    yield b("Second", "EE", "10115", "Tallinn", "Tartu mnt", 71)



from lino.api import rt

def objects():
    PartnerType = rt.models.actors.PartnerType
    Customers = rt.models.actors.Customers
    Providers = rt.models.actors.Providers
    
    yield PartnerType(
        id=rt.models.actors.Customers.partner_type_pk,
        name="Our customers")
    yield PartnerType(
        id=rt.models.actors.Providers.partner_type_pk,
        name="Our providers")

    yield Customers.make_instance(name="Adams")
    yield Customers.make_instance(name="Bowman")
    yield Providers.make_instance(name="Carlsson")
    yield Customers.make_instance(name="Dickens")



from django.db import models

from lino.api import dd
from lino.utils import join_elems
from etgen.html import E
from lino.mixins.polymorphic import Polymorphic


class Place(dd.Model):
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name



class Member(Polymorphic):
    name = models.CharField(max_length=200)
    place = dd.ForeignKey(Place, blank=True, null=True)
    email = models.EmailField(max_length=200, blank=True)

    def __str__(self):
        return self.name


class Customer(Member):
    customer_remark = models.CharField(max_length=200, blank=True)

    
class Supplier(Member):

    supplier_remark = models.CharField(max_length=200, blank=True)
    


class Product(dd.Model):
    name = models.CharField(max_length=200)

    suppliers = models.ManyToManyField(
        'Supplier', through='Offer',
        related_name='offered_products')
    customers = models.ManyToManyField(
        'Customer', through='Demand',
        related_name='wanted_products')

    def __str__(self):
        return self.name

    @dd.displayfield("Offered by")
    def offered_by(self, ar):
        if ar is None:
            return ''
        items = [ar.obj2html(o) for o in self.suppliers.all()]
        items = join_elems(items, sep=', ')
        return E.p(*items)

    @dd.displayfield("Wanted by")
    def demanded_by(self, ar):
        if ar is None:
            return ''
        items = [ar.obj2html(o) for o in self.customers.all()]
        items = join_elems(items, sep=', ')
        return E.p(*items)



class Offer(dd.Model):
    supplier = dd.ForeignKey(Supplier)
    product = dd.ForeignKey(Product)
    valid_until = models.DateField(blank=True, null=True)

    def __str__(self):
        return "%s offered by %s" % (self.product, self.supplier)



class Demand(dd.Model):
    customer = dd.ForeignKey(Customer)
    product = dd.ForeignKey(Product)

    def __str__(self):
        return "%s (%s)" % (self.product, self.customer)





import datetime

from ..settings import *

class Site(Site):
    project_name = 'apc'
    is_demo_site = True
    # ignore_dates_after = datetime.date(2019, 5, 22)
    the_demo_date = datetime.date(2015, 3, 12)

    # default_ui = "lino.modlib.extjs"
    # default_ui = "lino_react.react"

    def get_plugin_configs(self):
        yield super(Site, self).get_plugin_configs()
        yield ('ledger', 'start_year', 2014)


SITE = Site(globals())

DEBUG = True



# -*- coding: utf-8 -*-
# Copyright 2013-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Runs some tests about the disable-delete handler and cascading deletes.

Reproduces :ticket:`582`.

You can run only these tests by issuing::

  $ go apc
  $ python manage.py test tests.test_ddh

Or::

  $ python setup.py test -s tests.DemoTests.test_std

"""

from __future__ import unicode_literals
from __future__ import print_function
from builtins import str
import six

from django.core.exceptions import ValidationError
from lino.utils.djangotest import RemoteAuthTestCase
from lino.api import rt
from lino.utils.mti import delete_child
from lino.utils.instantiator import create_row


class DDHTests(RemoteAuthTestCase):
    maxDiff = None

    def test01(self):
        from lino.modlib.users.choicelists import UserTypes
        User = rt.models.users.User
        Partner = rt.models.contacts.Partner
        Person = rt.models.contacts.Person
        Company = rt.models.contacts.Company
        Role = rt.models.contacts.Role
        Account = rt.models.sepa.Account
        Invoice = rt.models.vat.VatAccountInvoice
        Journal = rt.models.ledger.Journal
        VoucherTypes = rt.models.ledger.VoucherTypes
        JournalGroups = rt.models.ledger.JournalGroups

        u = User(username='robin',
                 user_type=UserTypes.admin,
                 language="en")
        u.save()

        def createit():
            obj = Person(first_name="John", last_name="Doe")
            obj.full_clean()
            obj.save()
            pk = obj.pk
            return (obj, Partner.objects.get(pk=pk))

        #
        # If there are no vetos, user can ask to delete from any MTI form
        #
        pe, pa = createit()
        pe.delete()

        pe, pa = createit()
        pa.delete()

        #
        # Cascade-related objects (e.g. addresses) are deleted
        # independently of the polymorphic form which initiated
        # deletion.
        #

        def check_cascade(model):
            pe, pa = createit()
            obj = model.objects.get(pk=pa.pk)
            rel = Account(partner=pa, iban="AL32293653370340154130927280")
            rel.full_clean()
            rel.save()
            obj.delete()
            self.assertEqual(Account.objects.count(), 0)

        check_cascade(Partner)
        check_cascade(Person)

        #
        # Vetos of one form are deteced by all other forms.
        #
        def check_veto(obj, expected):
            try:
                obj.delete()
                self.fail("Failed to raise Warning({0})".format(expected))
            except Warning as e:
                self.assertEqual(str(e), expected)

        VKR = create_row(
            Journal,
            ref="VKR", name="VKR",
            voucher_type=VoucherTypes.get_for_model(Invoice),
            journal_group=JournalGroups.sales)
        
        pe, pa = createit()

        def check_vetos(obj, msg):
            m = obj.__class__
            obj.full_clean()
            obj.save()
            check_veto(pa, msg)
            check_veto(pe, msg)
            self.assertEqual(m.objects.count(), 1)
            obj.delete()
            self.assertEqual(m.objects.count(), 0)

        msg = "Cannot delete Partner Doe John because 1 Invoices refer to it."
        msg = "Kann Partner Doe John nicht löschen weil 1 Rechnungen darauf verweisen."
        check_vetos(Invoice(partner=pa, journal=VKR), msg)

        #
        # Having an invoice does not prevent from removing the Person
        # child of the partner.
        #
        
        invoice = create_row(Invoice, partner=pa, journal=VKR)
        self.assertEqual(Partner.objects.count(), 1)
        self.assertEqual(Person.objects.count(), 1)
        self.assertEqual(Invoice.objects.count(), 1)

        delete_child(pa, Person)

        # tidy up:
        self.assertEqual(Partner.objects.count(), 1)
        self.assertEqual(Person.objects.count(), 0)
        invoice.delete()
        pa.delete()
        self.assertEqual(Partner.objects.count(), 0)

        # But Lino refuses to remove the Person child if it has vetos.
        # For example if the person form is being used as a contact person.

        pe, pa = createit()
        co = create_row(Company, name="Test")
        create_row(Role, company=co, person=pe)
        msg = "[u'Cannot delete Partner Doe John because 1 Contact Persons refer to it.']"
        if six.PY2:
            msg = "[u'Kann Partner Doe John nicht l\\xf6schen weil 1 Kontaktpersonen darauf verweisen.']"
        else:
            msg = "['Kann Partner Doe John nicht löschen weil 1 Kontaktpersonen darauf verweisen.']"
        try:
            delete_child(pa, Person)
            self.fail("Expected ValidationError")
        except ValidationError as e:
            self.assertEqual(msg, str(e))




# -*- coding: UTF-8 -*-
# Copyright 2014-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)


"""
Default settings for an Estonian :ref:`cosi` site.

- `Estonian VAT rates
  <http://www.emta.ee/index.php?id=28460>`_


"""

from __future__ import unicode_literals

from lino_cosi.lib.cosi.settings import *

class Site(Site):
    languages = 'en et'

    demo_fixtures = 'std all_countries minimal_ledger \
    eesti furniture demo demo_bookings payments demo2'.split()

    # temporary:
    # demo_fixtures = 'std all_countries minimal_ledger \
    # furniture demo demo_bookings demo2'.split()

    #demo_fixtures = 'std few_countries minimal_ledger \
    #furniture \
    #demo demo_bookings payments demo2'.split()


    def get_plugin_configs(self):
        yield super(Site, self).get_plugin_configs()
        yield ('vat', 'declaration_plugin', 'lino_xl.lib.eevat')
        yield ('countries', 'hide_region', False)
        yield ('countries', 'country_code', 'EE')
        yield ('ledger', 'use_pcmn', True)

    # def get_installed_apps(self):
    #     yield super(Site, self).get_installed_apps()
    #     yield 'lino_xl.lib.eevat'
    #
    # def setup_plugins(self):
    #     "See :meth:`lino.core.site.Site.setup_plugins`."
    #     super(Site, self).setup_plugins()
    #     self.plugins.countries.configure(hide_region=False)
    #     self.plugins.ledger.configure(use_pcmn=True)
    #     self.plugins.countries.configure(country_code='EE')



import datetime
from django.utils import timezone
from django.db import models

from lino.api import dd



class Question(dd.Model):
    question_text = models.CharField("Question text", max_length=200)
    pub_date = models.DateTimeField('Date published', default=dd.today)
    hidden = models.BooleanField(
        "Hidden",
        help_text="Whether this poll should not be shown in the main window.",
        default=False)
    description = models.TextField('Description', blank=True)

    class Meta:
        verbose_name = 'Question'
        verbose_name_plural = 'Questions'

    def __str__(self):
        return self.question_text

    def was_published_recently(self):
        return self.pub_date >= timezone.now() - datetime.timedelta(days=1)

    def results_overview(self):
        lst = []
        total_votes = self.choice_set.aggregate(models.Sum('votes'))['votes__sum']
        for choice in self.choice_set.all():
            chunk = "{0}x {1}".format(choice.votes, choice)
            if total_votes:
                chunk += " ({0:.0f}%)".format(100.0 * choice.votes / total_votes)
            lst.append(chunk)
        s = " / ".join(lst)
        return s



class Choice(dd.Model):
    question = dd.ForeignKey(Question, on_delete=models.CASCADE)
    choice_text = models.CharField("Choice text", max_length=200)
    votes = models.IntegerField("No. of votes", default=0)

    class Meta:
        verbose_name = 'Choice'
        verbose_name_plural = 'Choices'

    def __str__(self):
        return self.choice_text

    @dd.action(help_text="Click here to vote this.")
    def vote(self, ar):
        def yes(ar):
            self.votes += 1
            self.save()
            return ar.success(
                "Thank you for voting %s" % self,
                "Voted!", refresh=True)
        if self.votes > 0:
            msg = "%s has already %d votes!" % (self, self.votes)
            msg += "\nDo you still want to vote for it?"
            return ar.confirm(yes, msg)
        return yes(ar)



# -*- coding: UTF-8 -*-
# Copyright 2014-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""

.. autosummary::
   :toctree:

   demo
   doctests
   www
   memory



"""

from __future__ import print_function
from __future__ import unicode_literals

# from lino_book.projects.team.settings import *
from lino_care.lib.care.settings import *
from lino.api.ad import _




# -*- coding: utf-8 -*-
# Copyright 2016-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Runs some tests about the disable-delete handler and cascading deletes.

You can run only these tests by issuing::

  $ go anna
  $ python manage.py test tests.test_cascaded_delete

This tests for :ticket:`1177`, :ticket:`1180`, :ticket:`1181`

"""

from __future__ import unicode_literals
from __future__ import print_function
import six

from django.core.exceptions import ValidationError
from lino.utils.djangotest import RemoteAuthTestCase
from lino.api import rt


def create(m, **kwargs):
    obj = m(**kwargs)
    obj.full_clean()
    obj.save()
    return obj
    

class Tests(RemoteAuthTestCase):
    maxDiff = None

    def test01(self):
        from lino.modlib.users.choicelists import UserTypes
        User = rt.models.users.User
        Person = rt.models.contacts.Person
        # Note = rt.models.notes.Note
        Skill = rt.models.skills.Skill
        Demand = rt.models.skills.Demand
        Competence = rt.models.skills.Competence

        general = create(Skill, name="General work")
        special = create(Skill, name="Special work", parent=general)

        alex = create(User, username='alex',
                      first_name="Alex",
                      user_type=UserTypes.user,
                      language="en")
        
        bruno = create(User, username='bruno',
                       first_name="Bruno",
                       user_type=UserTypes.user,
                       language="en")
        
        berta = create(User, username='berta',
                       first_name="Berta",
                       user_type=UserTypes.user,
                       language="en")
        
        claude = create(Person, first_name="Claude", language="en")
        
        # note1 = create(Note, user=berta, short_text="test 1")
        # note2 = create(Note, user=berta, short_text="test 2")
        note1 = create(Person, first_name="Dirk")
        note2 = create(Person, first_name="Eric")
        
        create(Competence, user=bruno, faculty=special)
        create(Competence, user=alex, faculty=general)
        
        create(Demand, demander=note1, skill=general)
        create(Demand, demander=note2, skill=special)

        ar = rt.models.skills.AssignableWorkersByTicket.request(note1)
        s = ar.to_rst()
        # print(s)
        self.assertEquivalent("""
==========
 Username
----------
 alex
==========
""", s)


        ar = rt.models.skills.AssignableWorkersByTicket.request(note2)
        s = ar.to_rst()
        # print(s)
        self.assertEquivalent("""
==========
 Username
----------
 alex
 bruno
==========
""", s)


        # cannot delete a faculty when there are competences referring
        # to it:
        try:
            special.delete()
            self.fail("Expected veto")
        except Warning as e:
            self.assertEqual(
                six.text_type(e), "Cannot delete Skill Special work "
                "because 1 Skill offers refer to it.")

        # you cannot delete a skill when it is the parent of other
        # skills
        try:
            general.delete()
            self.fail("Expected veto")
        except Warning as e:
            self.assertEqual(
                six.text_type(e), "Cannot delete Skill General work "
                "because 1 Skill offers refer to it.")
            
        # deleting a user will automatically delete all their
        # offers:
        
        bruno.delete()
        alex.delete()

        # make sure that database state is as expected:

        self.assertEqual(Skill.objects.count(), 2)
        self.assertEqual(User.objects.count(), 1)
        self.assertEqual(Person.objects.count(), 3)
        self.assertEqual(Competence.objects.count(), 0)
        
        



#!/usr/bin/env python
import os
import sys

if __name__ == "__main__":
    os.environ["DJANGO_SETTINGS_MODULE"] = "lino_book.projects.human.settings"

    from django.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)



# -*- coding: UTF-8 -*-
# Copyright 2012 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)


from lino.utils.instantiator import Instantiator, i2d
from lino.core.utils import resolve_model
from django.utils.translation import ugettext_lazy as _


from django.conf import settings
from lino.api import dd


def objects():

    mailType = Instantiator('notes.NoteType').build

    yield mailType(**dd.babel_values('name',
                                  en="Enrolment",
                                  fr=u'Inscription', de=u"Einschreibeformular"))
    yield mailType(**dd.babel_values('name',
                                  en="Timetable",
                                  fr=u'Horaire', de=u"Stundenplan"))
    yield mailType(**dd.babel_values('name',
                                  en="Letter",
                                  fr=u'Lettre', de=u"Brief"))



from builtins import object
from lino.api import dd
from lino import mixins
from django.utils.translation import ugettext_lazy as _


class Categories(dd.ChoiceList):
    verbose_name = _("Category")
    verbose_name_plural = _("Categories")

Categories.add_item("01", _("Hardware"), 'hardware')
Categories.add_item("02", _("Service"), 'service')
Categories.add_item("03", _("Accessories"), 'accessories')
Categories.add_item("04", _("Software"), 'software')


class Product(mixins.BabelNamed):

    price = dd.PriceField(_("Price"), blank=True, null=True)

    category = Categories.field(blank=True, null=True)

    class Meta(object):
        verbose_name = 'Product'
        verbose_name_plural = 'Products'


class Products(dd.Table):
    model = Product
    sort_order = ['name']
    column_names = "name category price *"
    auto_fit_column_widths = True

    detail_layout = """
    id price category
    name
    """

    insert_layout = dd.InsertLayout("""
    name
    category price
    """, window_size=(40, 'auto'))





# -*- coding: UTF-8 -*-

from __future__ import unicode_literals

from lino.api import dd, rt

Product = dd.resolve_model('babel_tutorial.Product')


def P(en, de, fr, cat):
    return Product(
        category=cat, **dd.babel_values('name', en=en, de=de, fr=fr))


def objects():
    yield P("Chair", "Stuhl", "Chaise", '03')
    yield P("Table", "Tisch", "Table", '03')
    yield P("Monitor", "Bildschirm", "Écran", '01')
    yield P("Mouse", "Maus", "Souris", '03')
    yield P("Keyboard", "Tastatur", "Clavier", '03')
    yield P("Consultation", "Beratung", "Consultation", '02')



# -*- coding: UTF-8 -*-
# Copyright 2015 Rumma & Ko Ltd
#
# This file is part of Lino Noi.
#
# Lino Noi is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Lino Noi is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with Lino Noi.  If not, see
# <http://www.gnu.org/licenses/>.

from lino.utils.djangotest import WebIndexTestCase

from lino.utils.djangotest import RemoteAuthTestCase


class TestCase(RemoteAuthTestCase):

    def test(self):

        self.assertEqual(1+1, 2)



# -*- coding: utf-8 -*-
# Copyright 2017-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""
Does not actually test anything.

You can run only these tests by issuing::

  $ python setup.py test -s tests.DemoTests.test_simple

Or::

  $ go adg
  $ python manage.py test tests.test_simple



"""

from __future__ import unicode_literals
from __future__ import print_function

from django.core.exceptions import ValidationError
from lino.utils.djangotest import RemoteAuthTestCase
from lino.api import rt


class SimpleTests(RemoteAuthTestCase):
    maxDiff = None

    def test01(self):
        User = rt.models.users.User
        UserTypes = rt.models.users.UserTypes




"""Default settings for a :mod:`lino_book.projects.dump` site.

This module instantiates a :setting:`SITE` variable and thus is
designed to be used directly as a :setting:`DJANGO_SETTINGS_MODULE`.

"""

from ..settings import *
from lino.utils import i2d


class Site(Site):
    languages = "en de fr"

    the_demo_date = i2d(20160702)

SITE = Site(globals())
# SECRET_KEY = "20227"  # see :djangoticket:`20227`
DEBUG = True




# -*- coding: UTF-8 -*-
# Copyright 2012-2013 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)



from lino.utils.instantiator import Instantiator, i2d
from django.utils.translation import ugettext_lazy as _

from lino.api import dd


def objects():

    #~ slot = Instantiator('courses.Slot','name start_time end_time').build
    #~
    #~ kw = dict(monday=True,tuesday=True,wednesday=False,thursday=True,friday=True)
    #~ yield slot("Erste Stunde","16:00","17:00",**kw)
    #~ yield slot("Zweite Stunde","17:00","18:00",**kw)
    #~ yield slot("Dritte Stunde","18:00","19:00",**kw)
    #~
    #~ kw = dict(wednesday=True)
    #~ yield slot("Mittwochs 13 Uhr","13:00","14:00",**kw)
    #~ yield slot("Mittwochs 14 Uhr","14:00","15:00",**kw)
    #~ yield slot("Mittwochs 15 Uhr","15:00","16:00",**kw)
    #~ yield slot("Mittwochs 16 Uhr","16:00","17:00",**kw)
    #~ yield slot("Mittwochs 17 Uhr","17:00","18:00",**kw)
    #~ yield slot("Mittwochs 18 Uhr","18:00","19:00",**kw)

    courses = dd.resolve_app('courses')

    yield courses.Line(**dd.babelkw('name',
                                 de=u"Deutsch Anfänger",
                                 fr=u"Allemand débutants",
                                 en=u"German beginners",
                                 ))
    yield courses.Line(**dd.babelkw('name',
                                 de=u"Französisch Anfänger",
                                 fr=u"Français débutants",
                                 en=u"French beginners",
                                 ))



# -*- coding: utf-8 -*-
# Copyright 2016-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Runs some tests about the notification framework.

You can run only these tests by issuing::

  $ go team
  $ python manage.py test tests.test_notify

Or::

  $ go book
  $ python setup.py test -s tests.test_demo.TestCase.test_team

"""

from __future__ import unicode_literals

# import six
import datetime

from mock import patch

from django.conf import settings
from django.utils.timezone import make_aware

from lino.api import dd, rt
from lino.utils.djangotest import TestCase
from lino.core import constants

from lino.modlib.users.choicelists import UserTypes

from lino.utils.instantiator import create

from lino.modlib.notify.models import send_pending_emails_often
from lino.modlib.notify.choicelists import MailModes
from lino.core.diff import ChangeWatcher

import sys
# from cStringIO import StringIO
from io import StringIO
import contextlib

@contextlib.contextmanager
def capture_stdout():
    oldout = sys.stdout
    try:
        out = StringIO()
        sys.stdout = out
        yield out
    finally:
        sys.stdout = oldout
        # out = out.getvalue()


class TestCase(TestCase):
    """Miscellaneous tests."""
    maxDiff = None

    def test_01(self):
        self.assertEqual(settings.SETTINGS_MODULE, None)
        self.assertEqual(settings.LOGGING, {})
        self.assertEqual(settings.SERVER_EMAIL, 'root@localhost')

    @patch('lino.api.dd.logger')
    def test_comment(self, logger):
        """Test what happens when a comment is posted on a ticket with
        watchers.

        """
        ContentType = rt.models.contenttypes.ContentType
        Ticket = rt.models.tickets.Ticket
        # Project = rt.models.tickets.Project
        Site = rt.models.tickets.Site
        Subscription = rt.models.tickets.Subscription
        # Vote = rt.models.votes.Vote
        # Star = rt.models.stars.Star
        Message = rt.models.notify.Message
        User = settings.SITE.user_model
        # create(Project, name="Project")
        robin = create(
            User, username='robin',
            first_name="Robin",
            user_type=UserTypes.admin)
        aline = create(
            User, username='aline',
            first_name="Aline",
            email="aline@example.com", language='fr',
            user_type=UserTypes.admin)

        foo = create(Site, name="Foo")
        create(Subscription, site=foo, user=aline)

        obj = create(
            Ticket, summary="Save the world, après moi le déluge",
            user=robin, site=foo)

        self.assertEqual(Message.objects.count(), 0)

        ar = rt.login('robin')
        self.client.force_login(ar.user)
        url = "/api/comments/CommentsByRFC"
        post_data = dict()
        post_data[constants.URL_PARAM_ACTION_NAME] = 'submit_insert'
        post_data.update(body="I don't agree (#foobar).")
        post_data[constants.URL_PARAM_MASTER_PK] = obj.pk
        ct = ContentType.objects.get_for_model(Ticket)
        post_data[constants.URL_PARAM_MASTER_TYPE] = ct.id
        # post_data[constants.URL_PARAM_REQUESTING_PANEL] = '123'
        self.client.force_login(robin)
        response = self.client.post(
            url, post_data,
            REMOTE_USER='robin',
            HTTP_ACCEPT_LANGUAGE='en')
        result = self.check_json_result(
            response, 'rows success message close_window navinfo')
        self.assertEqual(result['success'], True)
        self.assertEqual(
            result['message'],
            """Comment "Comment #1" has been created.""")

        self.assertEqual(Message.objects.count(), 1)
        msg = Message.objects.all()[0]
        # self.assertEqual(msg.message_type)
        self.assertEqual(msg.seen, None)
        self.assertEqual(msg.user, aline)
        expected = """Robin a commenté [ticket 1] (Save the world, """\
                   """après moi le déluge):<br>I don't agree (#foobar)."""
        self.assertEqual(expected, msg.body)

        # manually set created timestamp so we can test on it later.
        now = datetime.datetime(2016, 12, 22, 19, 45, 55)
        if settings.USE_TZ:
            now = make_aware(now)
        msg.created = now
        msg.save()

        settings.SERVER_EMAIL = 'root@example.com'

        with capture_stdout() as out:
            send_pending_emails_often()

        out = out.getvalue().strip()

        # if six.PY3:
        #     if isinstance(out, bytes):
        #         out = out.decode()

        # # if isinstance(out, bytes):
        # raise Exception(out)
        # print(out)

        expected = """send email
Sender: root@example.com
To: aline@example.com
Subject: [Django] Robin a comment? #1 (? Save the world, apr?s moi le d?luge)
<html><head><base href="http://127.0.0.1:8000/" target="_blank"></head><body>
(22/12/2016 19:45)
Robin a comment? <a href="/api/tickets/Tickets/1" title="Save the world, apr&#232;s moi le d&#233;luge">#1</a> (Save the world, apr?s moi le d?luge):<br>I don't agree (#foobar).
</body></html>
"""
        self.assertEquivalent(expected, out)

        self.assertEqual(logger.debug.call_count, 1)
        logger.debug.assert_called_with(
            'Send out %s summaries for %d users.',
            MailModes.often, 1)
        # logger.info.assert_called_with(
        #     'Notify %s users about %s', 1, 'Change by robin')

        Message.objects.all().delete()
        self.assertEqual(Message.objects.count(), 0)

        cw = ChangeWatcher(obj)
        from lino_xl.lib.tickets.choicelists import Priorities
        obj.priority = Priorities.low
        obj.save_watched_instance(ar, cw)


        with capture_stdout() as out:
            send_pending_emails_often()

        out = out.getvalue().strip()
        # print(out)
        expected = ""
        # self.assertEquivalent(expected, out)

        # we do not test the output because the datetime changes. But
        # we actually just wanted to see if there is no
        # UnicodeException. We capture it in order to hide it from
        # test runner output.

        self.assertEqual(logger.debug.call_count, 2)
        logger.debug.assert_called_with(
            'Send out %s summaries for %d users.',
            MailModes.often, 1)



# -*- coding: UTF-8 -*-
# Copyright 2013-2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""
The Lino Voga variant named "Roger".

.. autosummary::
   :toctree:

    lib
    settings
    settings.fixtures.voga
    tests.test_faggio
    tests.test_webindex
    tests.test_max_places

"""




from django.db import models
from django.utils.translation import ugettext_lazy as _

from lino.api import dd, rt


class A(dd.Action):
    label = _("a")

    def run_from_ui(self, ar, **kw):
        obj = ar.selected_rows[0]
        return ar.success("Called a() on %s" % obj)



class Moo(dd.Model):
    a = A()

    def __str__(self):
        return '%s object' % (self.__class__.__name__)

    @dd.action(_("m"))
    def m(self, ar, **kw):
        return ar.success("Called m() on %s" % self)


class Moos(dd.Table):
    model = Moo

    b = A()

    @dd.action(_("t"))
    def t(obj, ar, **kw):
        return ar.success("Called t() on %s" % obj)


class S1(Moos):
    pass


class S2(Moos):
    a = None
    b = None
    m = None
    t = None



# Copyright 2011-2014 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""
"""

from django.db import models
from django.utils.translation import ugettext_lazy as _


from lino.api import dd, rt
from lino_xl.lib.countries import models as countries
from lino_xl.lib.contacts import models as contacts

from lino.utils import str2hex, hex2str


class CRL(str):

    """\
A `Concise Reference Label`
    """
    pass


class CrlField(models.CharField):

    """A field that contains a Concise Reference Label.
CRL fields need to be sorted using pure ASCII sequence.
Since this is not a database-transparent feature in Django,
we store these strings as their hexadecimal representation.
    """

    def __init__(self, *args, **kw):
        defaults = dict(
            verbose_name=_("Label"),
            max_length=100,
            blank=True,
        )
        defaults.update(kw)
        models.CharField.__init__(self, *args, **defaults)

    def from_db_value(self, value, expression, connection, context=None):
        return CRL(hex2str(value)) if value else ''

    def to_python(self, value):
        if not value:
            return value
        if isinstance(value, CRL):
            return value
        return CRL(hex2str(value))

    def get_prep_value(self, value):
        if not value:
            return value
        assert isinstance(value, CRL)
        return str2hex(value)


dd.inject_field(countries.Place, 'crl', CrlField())
dd.inject_field(contacts.Person, 'crl', CrlField())
dd.inject_field(contacts.Person, 'died_date',
                models.DateField(
                    blank=True, null=True,
                    verbose_name=_("Died date")))
dd.inject_field(contacts.Company, 'crl', CrlField())



"""This is myscript.py, an usage example for the `run` management command.

Usage::

  $ manage.py run myscript NNN

where NNN is the id of some existing partner.

"""
import sys
from lino.api.shell import *
if len(sys.argv) != 2:
    print(__doc__)
    sys.exit(-1)
print(contacts.Partner.objects.get(pk=sys.argv[1]))



# -*- coding: UTF-8 -*-

from __future__ import unicode_literals
from __future__ import print_function

import datetime

from lino_voga.lib.voga.settings import *


class Site(Site):

    title = "Lino Voga à la Edmund"
    languages = "en et"

    # demo_fixtures = """std
    # few_countries few_cities
    # minimal_ledger euvatrates
    # demo voga demo2""".split()
    demo_fixtures = """std
    minimal_ledger
    demo voga demo2""".split()

    # ignore_dates_before = None
    the_demo_date = datetime.date(2014, 9, 26)
    # ignore_dates_after = datetime.date(2019, 05, 22)

    def setup_plugins(self):
        """
        Change the default value of certain plugin settings.
       
        """
        super(Site, self).setup_plugins()
        self.plugins.countries.configure(country_code='EE')
        self.plugins.ledger.configure(start_year=2014)



from django.db.models import Q
from lino.api import dd


class Places(dd.Table):
    model = 'lets.Place'


class Members(dd.Table):
    model = 'lets.Member'

    column_names = "name email place offered_products wanted_products"

    detail_layout = """
    id name place email
    OffersByMember DemandsByMember
    """


class Products(dd.Table):
    model = 'lets.Product'
    order_by = ['name']

    detail_layout = """
    id name
    OffersByProduct DemandsByProduct
    """

    column_names = 'id name providers customers'


class ActiveProducts(Products):

    label = "Active products"
    column_names = 'name offered_by wanted_by'

    @classmethod
    def get_request_queryset(cls, ar):
        # add filter condition to the queryset so that only active
        # products are shown, i.e. for which there is at least one
        # offer or one demand.
        qs = super(ActiveProducts, cls).get_request_queryset(ar)
        qs = qs.filter(Q(offer__isnull=False) | Q(demand__isnull=False))
        qs = qs.distinct()
        return qs


class Offers(dd.Table):
    model = 'lets.Offer'


class OffersByMember(Offers):
    master_key = 'provider'


class OffersByProduct(Offers):
    master_key = 'product'


class Demands(dd.Table):
    model = 'lets.Demand'


class DemandsByMember(Demands):
    master_key = 'customer'


class DemandsByProduct(Demands):
    master_key = 'product'




# -*- coding: UTF-8 -*-
# Copyright 2014-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Defines and instantiates a demo version of Lino Noi."""

import datetime

from ..settings import *


class Site(Site):

    the_demo_date = datetime.date(2015, 5, 23)

    languages = "en de fr"
    # readonly = True

    use_linod = True
    # use_ipdict = True
    # use_websockets = True
    social_auth_backends = [
        'social_core.backends.github.GithubOAuth2',
        # 'social_core.backends.google.GoogleOAuth2',
        # 'social_core.backends.google.GoogleOAuth',
        'social_core.backends.google.GooglePlusAuth',
        'social_core.backends.facebook.FacebookOAuth2',
        'social_core.backends.mediawiki.MediaWiki'
    ]
    use_experimental_features = True
    # default_ui = 'lino_extjs6.extjs6'
    # default_ui = 'lino.modlib.bootstrap3'
    # default_ui = 'lino_openui5.openui5'
    # default_ui = 'lino_react.react'

    # def get_installed_apps(self):
    #     yield super(Site, self).get_installed_apps()
    #     yield 'lino_react.react'

    def get_plugin_configs(self):
        for i in super(Site, self).get_plugin_configs():
            yield i
        yield ('excerpts', 'responsible_user', 'jean')
        # yield ('memo', 'front_end', 'react')

    def setup_plugins(self):
        """Change the default value of certain plugin settings.

        - :attr:`excerpts.responsible_user
          <lino_xl.lib.excerpts.Plugin.responsible_user>` is set to
          ``'jean'`` who is both senior developer and site admin in
          the demo database.

        """
        super(Site, self).setup_plugins()
        # self.plugins.social_auth.configure(
        #     backends=['social_core.backends.github.GithubOAuth2'])
        # self.plugins.excerpts.configure(responsible_user='jean')
        if False:
            self.plugins.mailbox.add_mailbox(
                'mbox', "Luc's aaa mailbox",
                '/home/luc/.thunderbird/luc/Mail/Local Folders/aaa')


SITE = Site(globals())

# SITE.plugins.extjs6.configure(theme_name='theme-classic')
# SITE.plugins.extjs6.configure(theme_name='theme-classic-sandbox')
# SITE.plugins.extjs6.configure(theme_name='theme-aria')
# SITE.plugins.extjs6.configure(theme_name='theme-grey')
# SITE.plugins.extjs6.configure(theme_name='theme-crisp')
# SITE.plugins.extjs6.configure(theme_name='theme-crisp-touch')
# SITE.plugins.extjs6.configure(theme_name='theme-neptune')
# SITE.plugins.extjs6.configure(theme_name='theme-neptune-touch')
# SITE.plugins.extjs6.configure(theme_name='theme-triton')
# SITE.plugins.extjs6.configure(theme_name='ext-theme-neptune-lino')

#in etc/aliases
# comments: /home/tonis/mbox
#SITE.plugins.inbox.configure(mbox_path='/home/tonis/mbox')
#SITE.plugins.inbox.configure(comment_reply_addr='comments@localhost')
DEBUG = True
ALLOWED_HOSTS=["*"]

# the following line should not be active in a checked-in version
#~ DATABASES['default']['NAME'] = ':memory:'

# SITE.update_settings(ALLOWED_HOSTS=["192.168.0.26","127.0.0.1"])

# https://github.com/organizations/lino-framework/settings/applications/632218
SOCIAL_AUTH_GITHUB_KEY = '355f66b1557f0cbf4d1d'
SOCIAL_AUTH_GITHUB_SECRET = '4dbeea1701bf03316c1759bdb422d9f88969b782'

SOCIAL_AUTH_GOOGLE_PLUS_KEY = '451271712409-9qtm9bvjndaeep2olk3useu61j6qu2kp.apps.googleusercontent.com'
SOCIAL_AUTH_GOOGLE_PLUS_SECRET = 'NHyaqV2HY8lV5ULG6k51OMwo'
# SOCIAL_AUTH_GOOGLE_OAUTH2_SCOPE = [
SOCIAL_AUTH_GOOGLE_PLUS_SCOPE = [
    'profile',
    'https://www.googleapis.com/auth/plus.login',
    'https://www.googleapis.com/auth/contacts.readonly', # To have just READ permission
    'https://www.googleapis.com/auth/contacts ', # To have WRITE/READ permissions
]

SOCIAL_AUTH_FACEBOOK_KEY = '1837593149865295'
SOCIAL_AUTH_FACEBOOK_SECRET = '1973f9e9d9420c4c6502aa40cb8cb7db'
SOCIAL_AUTH_FACEBOOK_SCOPE = ['email', 'public_profile', 'user_friends']

SOCIAL_AUTH_MEDIAWIKI_KEY = '7dbd2e1529e45108f798349811c7a2b7'
SOCIAL_AUTH_MEDIAWIKI_SECRET = '8041055fcd16333fa242b346e0ae52133fd2ee14'
SOCIAL_AUTH_MEDIAWIKI_URL = 'https://meta.wikimedia.org/w/index.php'
SOCIAL_AUTH_MEDIAWIKI_CALLBACK = 'oob'

if False:  # not needed for newbies

    # Add ldap authentication. Requires  Hamza's fork of django_auth_ldap.
    # temporary installation instructions:
    # $ sudo apt-get install build-essential python3-dev python2.7-dev libldap2-dev libsasl2-dev slapd ldap-utils lcov valgrind
    # $ pip install -e git+https://github.com/khchine5/django-auth-ldap.git#egg=django-auth-ldap
    # import ldap
    # from django_auth_ldap.config import LDAPSearch, LDAPGroupType,GroupOfNamesType,LDAPSearchUnion,GroupOfUniqueNamesType

    AUTHENTICATION_BACKENDS.append("django_auth_ldap.backend.LDAPBackend")

    AUTH_LDAP_SERVER_URI = "ldap://ldap.forumsys.com"
    AUTH_LDAP_USER_DN_TEMPLATE = "uid=%(user)s,dc=example,dc=com"
    AUTH_LDAP_USER_ATTR_MAP = {
        'first_name': 'givenName',
        'last_name': 'sn',
        'email': 'mail',
    }



# -*- coding: utf-8 -*-
# Copyright 2011 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

import logging
logger = logging.getLogger(__name__)

#~ from django.utils import unittest
#~ from django.test.client import Client
#from lino.igen import models
#from lino_xl.lib.contacts.models import Contact, Companies
#from lino_xl.lib.countries.models import Country
#~ from lino_xl.lib.contacts.models import Companies

from lino.core.utils import resolve_model
#Companies = resolve_model('contacts.Companies')
from lino.utils.djangotest import TestCase

#~ Person = resolve_model('contacts.Person')
Invoice = resolve_model('sales.Invoice')
#~ Property = resolve_model('properties.Property')
#~ PersonProperty = resolve_model('properties.PersonProperty')


class DemoTest(TestCase):
    #~ fixtures = [ 'std','demo' ]
    fixtures = 'std few_countries few_languages few_cities demo'.split()


def test01(self):
    """
    """
    self.assertEqual(Invoice.objects.all().count(), 28)
    i = Invoice.objects.all()[0]
    self.assertEqual(str(i), u"Invoice # 1")
    s = i.customer.address()
    self.assertEquals(s, u"""\
Rumma & Ko OÜ
Tartu mnt 71
10115 Tallinn
Estonia""")



import datetime

from ..settings import *


class Site(Site):
    is_demo_site = True
    the_demo_date = datetime.date(2015, 5, 22)
    # ignore_dates_after = datetime.date(2019, 05, 22)
    use_java = False
    # use_ipdict = True


SITE = Site(globals())
DEBUG = True

# the following line should not be active in a checked-in version
# DATABASES['default']['NAME'] = ':memory:'



from lino.api import dd, _
from lino.mixins.ref import Referrable


class MyMixin(dd.Model):

    class Meta:
        abstract = True

    @dd.displayfield(_("Foo"))
    def foo(self, ar):
        return str(self) + "!"


class MyModel(MyMixin):

    foo = dd.CharField(
        _("Foo"), max_length=200, blank=True)




from lino.projects.std.settings import *

# configure_plugin('countries', country_code='BE')

class Site(Site):

    verbose_name = "AddressLocation tutorial"

    demo_fixtures = ["few_countries", "few_cities", "demo"]

    def setup_plugins(self):
        super(Site, self).setup_plugins()
        self.plugins.countries.configure(country_code='BE')

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino_xl.lib.countries'
        yield 'lino_book.projects.addrloc'


SITE = Site(globals())
DEBUG = True



from lino.api import dd


class Companies(dd.Table):
    model = 'Company'
    column_names = 'name address_column *'
    detail_layout = dd.DetailLayout("""
    id name
    addr1
    street_prefix street street_no street_box
    addr2""", window_size=(50, 'auto'))



# -*- coding: UTF-8 -*-


__copyright__ = """\
Copyright (c) 2012-2013 Rumma & Ko Ltd.
This software comes with ABSOLUTELY NO WARRANTY and is
distributed under the terms of the GNU Lesser General Public License.
See file COPYING.txt for more information."""



# -*- coding: UTF-8 -*-
# Copyright 2013-2015 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
Default settings module for a :ref:`polly` project.
"""

from __future__ import unicode_literals

from lino.projects.std.settings import *

from django.utils.translation import ugettext_lazy as _


class Site(Site):

    """
    Base class for a :ref:`polly` application,
    designed to be instantiated into the :setting:`SITE` setting.
    """

    verbose_name = "Lino Polly"
    description = _("a Lino application to make surveys and polls.")
    version = "0.1"
    url = "http://www.lino-framework.org/examples/polly"
    author = 'Luc Saffre'
    author_email = 'luc.saffre@gmail.com'

    demo_fixtures = 'std demo feedback compass demo2'.split()
    # user_types_module = 'lino_book.projects.polly.user_types'
    user_types_module = 'lino_xl.lib.xl.user_types'

    languages = 'en de et'

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.gfks'
        yield 'lino.modlib.users'
        yield 'lino_xl.lib.polls'



# -*- coding: utf-8 -*-
# Copyright 2015-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Test some calendar functionalities-

This module is part of the Lino test suite. You can test only this
module by issuing either::

  $ go min2
  $ python manage.py test
  $ python manage.py test tests.test_cal.QuickTest

or::

  $ go lino
  $ python setup.py test -s tests.ProjectsTests.test_min2



"""

from __future__ import unicode_literals
from __future__ import print_function

from lino.api import dd, rt

from lino.utils.djangotest import RemoteAuthTestCase


class QuickTest(RemoteAuthTestCase):

    fixtures = ['std', 'demo_users']

    def test_create_entry(self):
        """# cal.MyEntries.insert({'requesting_panel': u'ext-comp-3913',
'user': u'luc'})

        """

        ses = rt.login("robin", renderer=dd.plugins.extjs.renderer)
        ba = rt.models.cal.MyEntries.get_action_by_name('submit_insert')
        # a = rt.models.cal.MyEntries.submit_insert
        # ba = rt.models.cal.MyEntries.insert_action
        pv = dict(user=ses.get_user())
        resp = ses.run(ba, param_values=pv)
        # ba.request_from(ses).run_from_ui(ses)
        self.assertEqual(sorted(resp.keys()), [
            'close_window', 'data_record', 'detail_handler_name',
            'info_message', 'message', 'refresh_all', 'rows', 'success'])

        # self.assertEqual(resp['data_record'].keys(), None)

        # The return message is of style 'Event "Event #69 (23.10.2014
        # 15:42)" has been created.'  The start_time is the real time
        # runtime and thus not predictable.  So we retrieve the
        # created object and use it to build that message:

        pk = resp['data_record']['id']
        obj = rt.models.cal.Event.objects.get(pk=pk)
        msg = 'Calendar entry "{0}" has been created.'.format(obj)
        self.assertEqual(resp['message'], msg)
        # self.assertEqual(msg, None)

        
        



# -*- coding: utf-8 -*-
# Copyright 2017-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
Tests some behaviour of the `Client.national_id` field.

You can run only these tests by issuing::

  $ go adg
  $ python manage.py test tests.test_nullchar

Tests whether `national_id` is set to NULL (not empty string).

"""

from __future__ import unicode_literals
from __future__ import print_function


import os
from six.moves.urllib.parse import urlencode

from lino.utils.djangotest import RemoteAuthTestCase
from lino.api import dd, rt
from lino.utils.instantiator import create_row


class TestCase(RemoteAuthTestCase):
    maxDiff = None
    # override_djangosite_settings = dict(use_java=True)

    def test01(self):
        from lino.core import constants
        from lino.modlib.users.choicelists import UserTypes
        from lino.api.shell import countries, users
        
        Client = rt.models.avanti.Client

        u = users.User(username='robin',
                       user_type=UserTypes.admin,
                       language="en")
        u.save()
        self.client.force_login(u)
        
        be = countries.Country(name="Belgium", isocode="BE")
        be.save()
        
        kw = dict()
        kw.update(national_id="680601 053-29")
        kw.update(first_name="Jean")
        kw.update(middle_name="Jacques")
        kw.update(last_name="Jeffin")
        jean = create_row(Client, **kw)

        kw.update(first_name="Jo")
        kw.update(national_id="680601 054-28")
        kw.update(last_name="Jeffin")
        jo = create_row(Client, **kw)

        def grid_put(username, url, **data):
            data[constants.URL_PARAM_ACTION_NAME] = 'grid_put'
            kwargs = dict(data=urlencode(data))
            kwargs['REMOTE_USER'] = username
            response = self.client.put(url, **kwargs)
            # print(response)
            return self.check_json_result(
                response, 'rows success message')

        url = '/api/avanti/Clients/' + str(jean.pk)
        result = grid_put('robin', url, national_id="")
        self.assertEqual(result['success'], True)
        self.assertEqual(result['message'], 'Client "JEFFIN Jean (100)" has been updated.')

        jean = Client.objects.get(pk=jean.pk)
        self.assertEqual(jean.national_id, None)
        
        url = '/api/avanti/Clients/' + str(jo.pk)
        result = grid_put('robin', url, national_id="")
        self.assertEqual(result['success'], True)
        self.assertEqual(result['message'], 'Client "JEFFIN Jo (101)" has been updated.')




# -*- coding: utf-8 -*-
# Copyright 2013-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)


"""This module contains tests that are run on a demo database without
any fixture. 

You can run only these tests by issuing::

  $ cd lino_book/projects/min2
  $ python manage.py test

"""

from __future__ import unicode_literals
from __future__ import print_function

import logging
logger = logging.getLogger(__name__)

from atelier.utils import AttrDict
import json

from lino.api.shell import *

from lino.utils.djangotest import RemoteAuthTestCase


class QuickTest(RemoteAuthTestCase):

    def test01(self):
        """
        Initialization.
        """
        #~ print "20130321 test00 started"
        self.user_root = settings.SITE.user_model(
            username='root', language='en', user_type='900')
        self.user_root.save()
        self.client.force_login(self.user_root)

        self.assertEqual(1 + 1, 2)
        o1 = contacts.Company(name="Example")
        o1.save()
        o2 = contacts.Company(name="Example")
        o2.save()

        p1 = contacts.Person(first_name="John", last_name="Doe")
        p1.full_clean()
        p1.save()
        p2 = contacts.Person(first_name="Johny", last_name="Doe")
        p2.full_clean()
        p2.save()

        contacts.Role(person=p1, company=o1).save()
        contacts.Role(person=p2, company=o2).save()

        evt = cal.Event()
        evt.full_clean()
        evt.save()
        guest = cal.Guest(event=evt, partner=p1)
        guest.full_clean()
        guest.save()

        # s = contacts.ContactsByOrganisation.request(o1).to_rst()
        s = contacts.RolesByCompany.request(o1).to_rst()
        # print('\n'+s)
        self.assertEqual(s, """\
========== ==============
 Person     Contact Role
---------- --------------
 John Doe
========== ==============

""")

        s = contacts.RolesByCompany.request(o2).to_rst()
        # print('\n'+s)
        self.assertEqual(s, """\
=========== ==============
 Person      Contact Role
----------- --------------
 Johny Doe
=========== ==============

""")
        # ba = contacts.Persons.get_action_by_name('merge_row')
        # self.assertEqual(ba, '')
        utpl = "/api/contacts/Persons/{0}?fv={1}&fv=&fv=false&fv=fff&an=merge_row"
        url = utpl.format(p1.pk, p1.pk)
        res = self.client.get(url, REMOTE_USER='root')
        self.assertEqual(res.status_code, 200)
        res = AttrDict(json.loads(res.content))
        self.assertEqual(res.message, "Cannot merge an instance to itself.")
        self.assertEqual(res.success, False)

        url = utpl.format(p1.pk, '')
        res = self.client.get(url, REMOTE_USER='root')
        self.assertEqual(res.status_code, 200)
        res = AttrDict(json.loads(res.content))
        self.assertEqual(res.message, "You must specify a merge target.")
        self.assertEqual(res.success, False)

        url = utpl.format(p1.pk, p2.pk)
        res = self.client.get(url, REMOTE_USER='root')
        self.assertEqual(res.status_code, 200)
        res = AttrDict(json.loads(res.content))
        # print(res)
        expected = '<div class="htmlText"><p>Are you sure you want to merge John Doe into Johny Doe?</p><ul><li>1 Contact Persons, 1 Presences <b>will get reassigned.</b></li><li>John Doe will be deleted</li></ul></div>'
        self.assertEqual(res.message, expected)
        self.assertEqual(res.success, True)
        self.assertEqual(res.close_window, True)
        self.assertEqual(res.xcallback['buttons'], {'yes': 'Yes', 'no': 'No'})
        self.assertEqual(res.xcallback['title'], "Confirmation")
        
        url = "/callbacks/{}/yes".format(res.xcallback['id'])
        res = self.client.get(url, REMOTE_USER='root')
        self.assertEqual(res.status_code, 200)
        res = AttrDict(json.loads(res.content))
        # print(res)
        self.assertEqual(
            res.message,
            'Merged John Doe into Johny Doe. Updated 2 related rows.')
        self.assertEqual(res.success, True)

        s = contacts.Roles.request().to_rst()
        # print('\n'+s)
        self.assertEqual(s, """\
==== ============== =========== ==============
 ID   Contact Role   Person      Organisation
---- -------------- ----------- --------------
 1                   Johny Doe   Example
 2                   Johny Doe   Example
==== ============== =========== ==============

""")

        s = cal.Guests.request().to_rst()
        # print('\n'+s)
        self.assertEqual(s, """\
=========== ====== ============= ======== ================================
 Partner     Role   Workflow      Remark   Calendar entry
----------- ------ ------------- -------- --------------------------------
 Doe Johny          **Invited**            Calendar entry #1 (23.10.2014)
=========== ====== ============= ======== ================================

""")


        # self.fail("TODO: execute a merge action using the web interface")

        # 20130418 server traceback caused when a pdf view of a table
        # was requested through the web interface.  TypeError:
        # get_handle() takes exactly 1 argument (2 given)
        url = settings.SITE.buildurl(
            'api/countries/Countries?cw=189&cw=45&cw=45&cw=36&ch=&ch=&ch=&ch=&ch=&ch=&ci=name&ci=isocode&ci=short_code&ci=iso3&name=0&an=as_pdf')
        msg = 'Using remote authentication, but no user credentials found.'
        if False:  # not converted after 20170609
            try:
                response = self.client.get(url)
                self.fail("Expected '%s'" % msg)
            except Exception as e:
                self.assertEqual(str(e), msg)

        # response = self.client.get(url, REMOTE_USER='foo')
        # self.assertEqual(response.status_code, 403,
        #                  "Status code for anonymous on GET %s" % url)
        from appy.pod import PodError

        """
        If oood is running, we get a 302, otherwise a PodError
        """
        try:
            response = self.client.get(url, REMOTE_USER='root')
            #~ self.assertEqual(response.status_code,200)
            result = self.check_json_result(response, 'success open_url')
            self.assertEqual(
                result['open_url'], "/media/cache/appypdf/127.0.0.1/countries.Countries.pdf")

        except PodError as e:
            pass
            #~ self.assertEqual(str(e), PodError: Extension of result file is "pdf".




# -*- coding: UTF-8 -*-
# Copyright 2015-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""The :xfile:`settings.py` modules for this variant.

It has :attr:`default_user<lino.core.site.Site.default_user>` set to
'anonymous', which causes it to deactivate both authentication and
sessions.

.. autosummary::
   :toctree:

   demo
   doctests

"""


from lino_book.projects.team.settings.demo import *


class Site(Site):

    default_ui = 'lino.modlib.bootstrap3'
    # default_user = 'anonymous'




# -*- coding: utf-8 -*-
# Copyright 2013-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)


"""This module contains tests that are run on a demo database without
any fixture. 

You can run only these tests by issuing::

  $ cd lino_book/projects/apc
  $ python manage.py test tests.test_ledger

"""

from __future__ import unicode_literals
from __future__ import print_function

from lino.api.shell import dd, ledger, settings

from lino.utils.djangotest import RemoteAuthTestCase


class QuickTest(RemoteAuthTestCase):

    def test01(self):
        self.assertEqual(dd.plugins.ledger.fix_y2k, False)
        self.assertEqual(settings.SITE.today().year, 2015)
        obj = ledger.FiscalYear.create_from_year(
            settings.SITE.today().year)
        obj.full_clean()
        obj.save()
        self.assertEqual(obj.ref, '2015')
        
        obj = ledger.AccountingPeriod()
        obj.full_clean()
        self.assertEqual(
            str(obj),
            'AccountingPeriod(start_date=2015-03-01,'
            'state=<PeriodStates.open:10>,year=1)')




# Generated by Django 2.2.4 on 2019-08-09 10:40

from django.db import migrations, models
import django.db.models.deletion
import lino.core.choicelists
import lino.core.fields
import lino.core.roles
import lino.modlib.users.choicelists
import lino.modlib.users.models
import lino.utils.mldbc.fields
import lino_xl.lib.xl.user_types


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contacts', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='e-mail address')),
                ('language', lino.utils.mldbc.fields.LanguageField(blank=True, choices=[('en', 'English'), ('de', 'German'), ('fr', 'French')], max_length=5, verbose_name='Language')),
                ('modified', models.DateTimeField(editable=False, help_text='The time when this database object was last modified.', null=True, verbose_name='Modified')),
                ('created', models.DateTimeField(editable=False, help_text='The time when this object was created.', verbose_name='Created')),
                ('start_date', models.DateField(blank=True, help_text='The site administrator can optionally specify a date when a\nuser started or stopped to be active.', null=True, verbose_name='Start date')),
                ('end_date', models.DateField(blank=True, null=True, verbose_name='End date')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, help_text='Not used in Lino.', null=True, verbose_name='last login')),
                ('username', models.CharField(help_text='Must be unique and cannot be empty.', max_length=30, unique=True, verbose_name='Username')),
                ('user_type', lino.core.choicelists.ChoiceListField(blank=True, choicelist=lino.modlib.users.choicelists.UserTypes, choices=[(lino.modlib.users.choicelists.UserType('000', 'Anonymous', lino.core.roles.UserRole, name='anonymous', readonly=True), '000 (000 (Anonymous))'), (lino.modlib.users.choicelists.UserType('100', 'User', lino_xl.lib.xl.user_types.SiteUser, name='user'), '100 (100 (User))'), (lino.modlib.users.choicelists.UserType('900', 'Administrator', lino_xl.lib.xl.user_types.SiteAdmin, name='admin'), '900 (900 (Administrator))')], help_text='The user_type of a user is what defines her or his permissions.', max_length=20, verbose_name='User type')),
                ('initials', models.CharField(blank=True, help_text='The nickname or initials of this user. This does not need to\nbe unique but should provide a reasonably identifying\nfunction.', max_length=10, verbose_name='Initials')),
                ('first_name', models.CharField(blank=True, max_length=30, verbose_name='First name')),
                ('last_name', models.CharField(blank=True, max_length=30, verbose_name='Last name')),
                ('remarks', models.TextField(blank=True, verbose_name='Remarks')),
                ('partner', models.ForeignKey(blank=True, help_text='Pointer to the Partner instance related to\nthis user.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='users_by_partner', to='contacts.Partner')),
            ],
            options={
                'verbose_name': 'User',
                'verbose_name_plural': 'Users',
                'ordering': ['last_name', 'first_name', 'username'],
                'abstract': False,
            },
            bases=(models.Model, lino.core.fields.TableRow),
            managers=[
                ('objects', lino.modlib.users.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='Authority',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('authorized', models.ForeignKey(help_text='The user who gets the right to represent the author', on_delete=django.db.models.deletion.PROTECT, to='users.User')),
                ('user', models.ForeignKey(blank=True, help_text='The user who gives the right of representation. author of this\nauthority', on_delete=django.db.models.deletion.PROTECT, related_name='users_authority_set_by_user', to='users.User', verbose_name='Author')),
            ],
            options={
                'verbose_name': 'Authority',
                'verbose_name_plural': 'Authorities',
            },
            bases=(models.Model, lino.core.fields.TableRow),
        ),
    ]



# Copyright 2012-2015 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from django.db import models
from django.utils.translation import ugettext_lazy as _


from lino.api import dd, rt
from lino import mixins
#~ from lino.models import SiteConfig

#~ from lino_xl.lib.contacts import models as contacts
#~ from lino_xl.lib.cal import models as cal

contacts = dd.resolve_app('contacts')
#~ cal = dd.resolve_app('cal')
#~ school = dd.resolve_app('school')


class School(contacts.Company):

    class Meta:
        #~ app_label = 'school'
        verbose_name = _("School")
        verbose_name_plural = _("Schools")


class Schools(contacts.Companies):
    model = School


class Person(contacts.Person, mixins.Born):

    class Meta(contacts.Person.Meta):
        app_label = 'contacts'
        # ~ # see :srcref:`docs/tickets/14`
        #~ verbose_name = _("Person")
        #~ verbose_name_plural = _("Persons")


class PersonDetail(contacts.PersonDetail):

    #~ contact = contacts.PersonDetail.main
    #~ outbox = dd.Panel("""
    #~ outbox.MailsByProject
    #~ """,label = _("Correspondence"))
    #~ calendar = dd.Panel("""
    #~ cal.EntriesByProject
    #~ cal.TasksByProject
    #~ """,label = _("Calendar"))
    #~ main = "contact outbox calendar"
    main = """
    box1 box2
    remarks contacts.RolesByPerson households.MembersByPerson
    """

    box1 = """
    last_name first_name:15 #title:10
    country city zip_code:10
    #street_prefix street:25 street_no street_box
    addr2:40
    is_pupil is_teacher
    """

    box2 = """
    id:12 language
    email
    phone fax
    gsm
    gender birth_date age:10 
    """


    #~ def setup_handle(self,lh):

        #~ lh.contact.label = _("Contact")
        #~ lh.mails.label = _("Mails")


#~ class Company(contacts.Partner,contacts.CompanyMixin):
    #~ class Meta(contacts.CompanyMixin.Meta):
        #~ app_label = 'contacts'
        # ~ # see :srcref:`docs/tickets/14`
        #~ verbose_name = _("Company")
        #~ verbose_name_plural = _("Companies")
#~ class Event(cal.Event):
    #~ class Meta(cal.Event.Meta):
        #~ app_label = 'cal'
#~ class Task(cal.Task):
    #~ class Meta(cal.Task.Meta):
        #~ app_label = 'cal'
#~ class EventDetail(cal.EventDetail):
#~ class EventDetail(dd.FormLayout):
    #~ main = "general more"
    #~ lesson = dd.Panel("""
    #~ owner start_date start_time end_time place
    #~ school.PresencesByEvent
    #~ """,label=_("Lesson"))
    #~ event = dd.Panel("""
    # ~ id:8 user priority access_class transparent #rset
    #~ summary state workflow_buttons
    #~ calendar created:20 modified:20
    #~ description
    #~ cal.GuestsByEvent
    #~ """,label=_("Event"))
    #~ main = "lesson event"
    #~ def setup_handle(self,lh):
        #~ lh.lesson.label = _("Lesson")
        #~ lh.event.label =
        #~ lh.notes.label = _("Notes")

@dd.receiver(dd.post_analyze)
def my_details(sender, **kw):
    site = sender

    site.modules.cal.Events.set_detail_layout('general more')
    site.modules.cal.Events.add_detail_panel('general', """
    event_type summary user project 
    start end 
    room priority access_class transparent #rset 
    owner workflow_buttons
    description cal.GuestsByEvent 
    """, _("General"))

    site.modules.cal.Events.add_detail_panel('more', """
    id created:20 modified:20  
    outbox.MailsByController #postings.PostingsByController
    """, _("More"))

    # remove `project` field
    #~ site.modules.cal.Tasks.set_detail_layout("""
    #~ start_date workflow_buttons due_date done user id
    #~ summary
    #~ calendar owner created:20 modified:20 user_modified
    # ~ description #notes.NotesByTask
    #~ """)
    #~ site.modules.cal.Events.set_detail_layout("general more")
    site.modules.cal.Events.set_insert_layout("""
    summary 
    start end 
    event_type project 
    """,
                                              start="start_date start_time",
                                              end="end_date end_time",
                                              window_size=(60, 'auto'))


# TODO : move to plugin
def setup_main_menu(config, site, user_type, main):
    m = main.get_item("contacts")
    m.add_action('homeworkschool.Schools')


def customize_school():
    dd.inject_field('courses.Pupil',
                    'school',
                    dd.ForeignKey(School,
                                      blank=True, null=True,
            help_text=_("""The regular school where this child goes.""")
                    ))

customize_school()
